from base64 import b64encode, urlsafe_b64decode
from time import time as timestamp
from typing import Dict, List
import base64
import hashlib
import hmac
from uuid import uuid4


# tapjoy = "https://ads.tapdaq.com/v4/analytics/reward"
webApi = "https://aminoapps.com/api{}".format
api = "https://service.aminoapps.com/api/v1{}".format


def generateSig(data: str):
    return base64.b64encode(
        bytes.fromhex("19") + hmac.new(bytes.fromhex("dfa5ed192dda6e88a12fe12130dc6206b1251e44"),
        data.encode(),
        hashlib.sha1).digest()
    ).decode()

def generateDevice():
    data = uuid4().bytes
    return (
        "19" + data.hex() +
        hmac.new(bytes.fromhex("e7309ecc0953c6fa60005b2765f99dbbc965c8e9"),
        bytes.fromhex("19") + data,
        hashlib.sha1).hexdigest()
        ).upper()

def uuidString():
    return str(uuid4())


def active_time(seconds=0, minutes=5, hours=0) -> List[Dict[str, int]]:
    total = seconds + minutes*60 + hours*60*60
    return [
        {
            'start': int(timestamp()),
            'end': int(timestamp() + 300)
        } for _ in range(total // 300)
    ] + [
        {
            'start': int(timestamp()),
            'end': int(timestamp() + total % 300)
        }
    ]


def decode_sid(sid: str) -> dict:
    return json.loads(urlsafe_b64decode(sid + "=" * (4 - len(sid) % 4))[1:-20])

def sid_to_uid(SID: str) -> str: return decode_sid(SID)["2"]

def sid_to_ip_address(SID: str) -> str: return decode_sid(SID)["4"]

def sid_created_time(SID: str) -> str: return decode_sid(SID)["5"]

def sid_to_client_type(SID: str) -> str: return decode_sid(SID)["6"]


from typing import BinaryIO, Union

from httpx import Client
from json_minify import json_minify
from ujson import dumps

from .exception import CheckExceptions
from .headers import Headers
from .util import *

user_settings = {
    "sid": None,
    "userId": None,
    "secret": None
}

class Session(Headers):
    def __init__(self, proxies: Union[dict, str] = None, staticDevice: str = None):
        self.proxy = proxies
        self.staticDevice = staticDevice

        self.sid = user_settings["sid"]
        self.uid = user_settings["userId"]
        self.secret = user_settings["secret"]

        Headers.__init__(self, header_device=self.staticDevice)
        self.session = Client(proxies=self.proxy, timeout = 20)

        self.deviceId = self.header_device
        self.sidInit()

    def sidInit(self):
        if self.sid: self.updateHeaders(sid = self.sid)

    def settings(self, user_session: str = None, user_userId: str = None, user_secret: str = None):
        user_settings.update({
            "sid": user_session,
            "userId": user_userId,
            "secret": user_secret
        })

        self.sid = user_settings["sid"]
        self.uid = user_settings["userId"]
        self.secret = user_settings["secret"]

        self.sidInit()

    def postRequest(self, url: str, data: Union[str, dict, BinaryIO] = None, newHeaders: dict = None, webRequest: bool = False, minify: bool = False):
        if newHeaders: self.app_headers.update(newHeaders)

        if not isinstance(data, (str, BinaryIO)):
            data = json_minify(dumps(data)) if minify else dumps(data)

        req = self.session.post(
            url=webApi(url) if webRequest else api(url),
            data=data if isinstance(data, str) else None,
            files={"file": data} if isinstance(data, BinaryIO) else None,
            headers=self.web_headers if webRequest else self.updateHeaders(data=data, sid=self.sid)
        )
        return CheckExceptions(req.json()) if req.status_code != 200 else req.json()

    def getRequest(self, url: str):
        req = self.session.get(url=api(url), headers=self.updateHeaders())
        return CheckExceptions(req.json()) if req.status_code != 200 else req.json()

    def deleteRequest(self, url: str):
        req = self.session.delete(url=api(url), headers=self.updateHeaders())
        return CheckExceptions(req.json()) if req.status_code != 200 else req.json()



from time import time as timestamp

from ..lib.objects import *
from ..lib.async_sessions import AsyncSession as Session


class AsyncAcm(Session):
    def __init__(self, comId: str, proxies: dict = None):
        self.comId = comId
        self.proxies = proxies

        Session.__init__(self, proxies=self.proxies)

    async def promote(self, userId: str, rank: str):
        rank = rank.lower().replace("agent", "transfer-agent")

        if rank not in ["transfer-agent", "leader", "curator"]:
            raise TypeError(rank)

        req = await self.postRequest(f"/x{self.comId}/s/user-profile/{userId}/{rank}")
        return Json(req)

    async def accept_join_request(self, userId: str):
        req = await self.postRequest(
            f"/x{self.comId}/s/community/membership-request/{userId}/accept"
        )
        return Json(req)

    async def reject_join_request(self, userId: str):
        req = await self.postRequest(
            f"/x{self.comId}/s/community/membership-request/{userId}/reject"
        )
        return Json(req)

    async def change_welcome_message(self, message: str, enabled: bool = True):
        data = {
            "path": "general.welcomeMessage",
            "value": {"enabled": enabled, "text": message},
            "timestamp": int(timestamp() * 1000),
        }
        req = await self.postRequest(f"/x{self.comId}/s/community/configuration", data)
        return Json(req)

    async def change_guidelines(self, content: str):
        data = {"content": content, "timestamp": int(timestamp() * 1000)}
        req = await self.postRequest(f"/x{self.comId}/s/community/guideline", data)
        return Json(req)

    async def edit_community(
        self,
        name: str = None,
        description: str = None,
        aminoId: str = None,
        language: str = None,
        themePackUrl: str = None,
    ):
        data = {"timestamp": int(timestamp() * 1000)}

        if name:
            data["name"] = name
        if description:
            data["content"] = description
        if aminoId:
            data["endpoint"] = aminoId
        if language:
            data["primaryLanguage"] = language
        if themePackUrl:
            data["themePackUrl"] = themePackUrl

        req = await self.postRequest(f"/x{self.comId}/s/community/settings", data)
        return Json(req)

    async def get_community_stats(self):
        req = await self.getRequest(f"/x{self.comId}/s/community/stats")
        return CommunityStats(req["communityStats"]).CommunityStats

    async def get_admin_stats(self, moderationType: str, start: int = 0, size: int = 25):
        if moderationType not in ["leader", "curator"]:
            raise TypeError(moderationType)

        req = await self.getRequest(
            f"/x{self.comId}/s/community/stats/moderation?type={moderationType}&start={start}&size={size}"
        )
        return Json(req)

    async def get_join_requests(self, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/community/membership-request?status=pending&start={start}&size={size}"
        )
        return JoinRequest(req).JoinRequest

    async def get_all_members(self, usersType: str, start: int = 0, size: int = 25):
        usersType = usersType.lower()
        req = await self.getRequest(
            f"/x{self.comId}/s/user-profile?type={usersType}&start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    async def add_influencer(self, userId: str, monthlyFee: int = 50):
        data = {"monthlyFee": monthlyFee, "timestamp": int(timestamp() * 1000)}
        req = await self.postRequest(f"/x{self.comId}/s/influencer/{userId}", data)
        return Json(req)

    async def remove_influencer(self, userId: str):
        req = await self.deleteRequest(f"/x{self.comId}/s/influencer/{userId}")
        return Json(req)



from json import loads
from requests import get

from .k_sync.acm import Acm
from .k_sync.client import Client
from .k_sync.local import SubClient
# from .k_sync.sockets import *
from .k_sync.bot import Bot

from .k_async.acm import AsyncAcm
from .k_async.client import AsyncClient
from .k_async.local import AsyncSubClient
# from .k_async.sockets import *
from .k_async.bot import AsyncBot


__version__ = "1.2.0"


__newest__ = loads(get("https://pypi.python.org/pypi/k-amino.py/json").text)["info"]["version"]

if __version__ != __newest__:
    print(f"\033[1;31;38mk_amino New Version!: {__newest__} (You are using {__version__})\033[1;36;33m\nDiscord server: \"https://discord.gg/zd8gyFJquT\"\033[1;0m")
else:
    print(f"\033[1;31;32mk_amino version : {__version__}\033[1;36;33m\nDiscord server: \"https://discord.gg/zd8gyFJquT\"\033[1;0m")



# From https://github.com/okok7711/AminoAcid/blob/master/aminoacid/exceptions.py


class AminoBaseException(Exception):
    ...


class UnknownExcepion(AminoBaseException):
    ...


class AccessDenied(AminoBaseException):
    ...


class UnsupportedService(AminoBaseException):
    ...


class InvalidRequest(AminoBaseException):
    ...


class InvalidSession(AminoBaseException):
    ...


class InvalidAccountOrPassword(AminoBaseException):
    ...


class InvalidDevice(AminoBaseException):
    ...


class TooManyRequests(AminoBaseException):
    ...


class ActionNotAllowed(AminoBaseException):
    ...


class FileTooLarge(AminoBaseException):
    ...


class UnexistentData(AminoBaseException):
    ...


class MessageNeeded(AminoBaseException):
    ...


class AccountDisabled(AminoBaseException):
    ...


class InvalidEmail(AminoBaseException):
    ...


class InvalidPassword(AminoBaseException):
    ...


class EmailAlreadyTaken(AminoBaseException):
    ...


class AccountDoesntExist(AminoBaseException):
    ...


class CantFollowYourself(AminoBaseException):
    ...


class UserUnavailable(AminoBaseException):
    ...


class YouAreBanned(AminoBaseException):
    ...


class UserNotMemberOfCommunity(AminoBaseException):
    ...


class RequestRejected(AminoBaseException):
    ...


class ActivateAccount(AminoBaseException):
    ...


class CantLeaveCommunity(AminoBaseException):
    ...


class ReachedTitleLength(AminoBaseException):
    ...


class EmailFlaggedAsSpam(AminoBaseException):
    ...


class AccountDeleted(AminoBaseException):
    ...


class API_ERR_EMAIL_NO_PASSWORD(AminoBaseException):
    ...


class API_ERR_COMMUNITY_USER_CREATED_COMMUNITIES_VERIFY(AminoBaseException):
    ...


class ReachedMaxTitles(AminoBaseException):
    ...


class VerificationRequired(AminoBaseException):
    ...


class API_ERR_INVALID_AUTH_NEW_DEVICE_LINK(AminoBaseException):
    ...


class CommandCooldown(AminoBaseException):
    ...


class UserBannedByTeamAmino(AminoBaseException):
    ...


class BadImage(AminoBaseException):
    ...


class InvalidThemepack(AminoBaseException):
    ...


class InvalidVoiceNote(AminoBaseException):
    ...


class RequestedNoLongerExist(AminoBaseException):
    ...


class PageRepostedTooRecently(AminoBaseException):
    ...


class InsufficientLevel(AminoBaseException):
    ...


class WallCommentingDisabled(AminoBaseException):
    ...


class CommunityNoLongerExists(AminoBaseException):
    ...


class InvalidCodeOrLink(AminoBaseException):
    ...


class CommunityNameAlreadyTaken(AminoBaseException):
    ...


class CommunityCreateLimitReached(AminoBaseException):
    ...


class CommunityDisabled(AminoBaseException):
    ...


class CommunityDeleted(AminoBaseException):
    ...


class ReachedMaxCategories(AminoBaseException):
    ...


class DuplicatePollOption(AminoBaseException):
    ...


class ReachedMaxPollOptions(AminoBaseException):
    ...


class TooManyChats(AminoBaseException):
    ...


class ChatFull(AminoBaseException):
    ...


class TooManyInviteUsers(AminoBaseException):
    ...


class ChatInvitesDisabled(AminoBaseException):
    ...


class RemovedFromChat(AminoBaseException):
    ...


class UserNotJoined(AminoBaseException):
    ...


class API_ERR_CHAT_VVCHAT_NO_MORE_REPUTATIONS(AminoBaseException):
    ...


class MemberKickedByOrganizer(AminoBaseException):
    ...


class LevelFiveRequiredToEnableProps(AminoBaseException):
    ...


class ChatViewOnly(AminoBaseException):
    ...


class ChatMessageTooBig(AminoBaseException):
    ...


class InviteCodeNotFound(AminoBaseException):
    ...


class AlreadyRequestedJoinCommunity(AminoBaseException):
    ...


class API_ERR_PUSH_SERVER_LIMITATION_APART(AminoBaseException):
    ...


class API_ERR_PUSH_SERVER_LIMITATION_COUNT(AminoBaseException):
    ...


class API_ERR_PUSH_SERVER_LINK_NOT_IN_COMMUNITY(AminoBaseException):
    ...


class API_ERR_PUSH_SERVER_LIMITATION_TIME(AminoBaseException):
    ...


class AlreadyCheckedIn(AminoBaseException):
    ...


class AlreadyUsedMonthlyRepair(AminoBaseException):
    ...


class AccountAlreadyRestored(AminoBaseException):
    ...


class IncorrectVerificationCode(AminoBaseException):
    ...


class NotOwnerOfChatBubble(AminoBaseException):
    ...


class NotEnoughCoins(AminoBaseException):
    ...


class AlreadyPlayedLottery(AminoBaseException):
    ...


class CannotSendCoins(AminoBaseException):
    ...


class AminoIDAlreadyChanged(AminoBaseException):
    ...


class InvalidAminoID(AminoBaseException):
    ...


class InvalidName(AminoBaseException):
    ...


def CheckExceptions(data: dict):
    raise {
            100: UnsupportedService,
            102: FileTooLarge,
            103: InvalidRequest,
            104: InvalidRequest,
            105: InvalidSession,
            106: AccessDenied,
            107: UnexistentData,
            110: ActionNotAllowed,
            113: MessageNeeded,
            200: InvalidAccountOrPassword,
            201: AccountDisabled,
            210: AccountDisabled,
            213: InvalidEmail,
            214: InvalidPassword,
            215: EmailAlreadyTaken,
            216: AccountDoesntExist,
            218: InvalidDevice,
            219: TooManyRequests,
            221: CantFollowYourself,
            225: UserUnavailable,
            229: YouAreBanned,
            230: UserNotMemberOfCommunity,
            235: RequestRejected,
            238: ActivateAccount,
            239: CantLeaveCommunity,
            240: ReachedTitleLength,
            241: EmailFlaggedAsSpam,
            246: AccountDeleted,
            251: API_ERR_EMAIL_NO_PASSWORD,
            257: API_ERR_COMMUNITY_USER_CREATED_COMMUNITIES_VERIFY,
            262: ReachedMaxTitles,
            270: VerificationRequired,
            271: API_ERR_INVALID_AUTH_NEW_DEVICE_LINK,
            291: CommandCooldown,
            293: UserBannedByTeamAmino,
            300: BadImage,
            313: InvalidThemepack,
            314: InvalidVoiceNote,
            500: RequestedNoLongerExist,
            700: RequestedNoLongerExist,
            1600: RequestedNoLongerExist,
            503: PageRepostedTooRecently,
            551: InsufficientLevel,
            702: WallCommentingDisabled,
            801: CommunityNoLongerExists,
            802: InvalidCodeOrLink,
            805: CommunityNameAlreadyTaken,
            806: CommunityCreateLimitReached,
            814: CommunityDisabled,
            833: CommunityDeleted,
            1002: ReachedMaxCategories,
            1501: DuplicatePollOption,
            1507: ReachedMaxPollOptions,
            1602: TooManyChats,
            1605: ChatFull,
            1606: TooManyInviteUsers,
            1611: ChatInvitesDisabled,
            1612: RemovedFromChat,
            1613: UserNotJoined,
            1627: API_ERR_CHAT_VVCHAT_NO_MORE_REPUTATIONS,
            1637: MemberKickedByOrganizer,
            1661: LevelFiveRequiredToEnableProps,
            1663: ChatViewOnly,
            1664: ChatMessageTooBig,
            1900: InviteCodeNotFound,
            2001: AlreadyRequestedJoinCommunity,
            2501: API_ERR_PUSH_SERVER_LIMITATION_APART,
            2502: API_ERR_PUSH_SERVER_LIMITATION_COUNT,
            2503: API_ERR_PUSH_SERVER_LINK_NOT_IN_COMMUNITY,
            2504: API_ERR_PUSH_SERVER_LIMITATION_TIME,
            2601: AlreadyCheckedIn,
            2611: AlreadyUsedMonthlyRepair,
            2800: AccountAlreadyRestored,
            3102: IncorrectVerificationCode,
            3905: NotOwnerOfChatBubble,
            4300: NotEnoughCoins,
            4400: AlreadyPlayedLottery,
            4500: CannotSendCoins,
            4501: CannotSendCoins,
            6001: AminoIDAlreadyChanged,
            6002: InvalidAminoID,
            9901: InvalidName,
        }.get(data.get("api:statuscode", 1000), UnknownExcepion)(data)



import os
from base64 import b64encode
from binascii import hexlify
from time import time as timestamp
from time import timezone as timezones
from typing import Union, BinaryIO
from uuid import UUID

from ..lib.objects import *
from ..lib.sessions import Session
from .acm import Acm


class SubClient(Acm, Session):
    def __init__(self, comId: str, proxies: dict = None, acm: bool = False):
        self.proxies = proxies
        self.comId = comId

        if acm:
            Acm.__init__(self, comId=self.comId, proxies=self.proxies)
        else:
            Session.__init__(self, proxies=self.proxies)

    def get_video_rep_info(self, chatId: str):
        req = self.getRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/avchat-reputation"
        )
        return RepInfo(req)

    def claim_video_rep(self, chatId: str):
        req = self.postRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/avchat-reputation"
        )
        return RepInfo(req)

    def join_chat(self, chatId: str = None):
        req = self.postRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.uid}"
        )
        return Json(req)

    def upload_media(self, file: BinaryIO, fileType: str):
        if fileType == "audio":
            fileType = "audio/aac"
        elif fileType == "image":
            fileType = "image/jpg"
        else:
            raise TypeError(fileType)

        data = file.read()
        newHeaders = {"content-type": fileType, "content-length": str(len(data))}

        req = self.postRequest("/g/s/media/upload", data=data, newHeaders=newHeaders)
        return req["mediaValue"]

    def leave_chat(self, chatId: str = None):
        req = self.deleteRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.uid}"
        )
        return Json(req)

    def get_member_following(self, userId: str = None, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/user-profile/{userId}/joined?start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    def get_member_followers(self, userId: str = None, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/user-profile/{userId}/member?start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    def get_chat_threads(self, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/chat/thread?type=joined-me&start={start}&size={size}"
        )
        return ThreadList(req["threadList"]).ThreadList

    def get_member_visitors(self, userId: str, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/user-profile/{userId}/visitors?start={start}&size={size}"
        )
        return VisitorsList(req["visitors"]).VisitorsList

    def get_chat_messages(self, chatId: str, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/message?v=2&pagingType=t&size={size}"
        )
        return MessageList(req["messageList"]).MessageList

    def get_user_info(self, userId: str):
        req = self.getRequest(f"/x{self.comId}/s/user-profile/{userId}")
        return UserProfile(req["userProfile"]).UserProfile

    def get_user_blogs(self, userId: str, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/blog?type=user&q={userId}&start={start}&size={size}"
        )
        return BlogList(req["blogList"]).BlogList

    def get_user_wikis(self, userId: str, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/item?type=user-all&start={start}&size={size}&cv=1.2&uid={userId}"
        )
        return WikiList(req["itemList"]).WikiList

    def get_all_users(self, usersType: str = "recent", start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/user-profile?type={usersType}&start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    def get_chat_members(self, start: int = 0, size: int = 25, chatId: str = None):
        req = self.getRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/member?start={start}&size={size}&type=default&cv=1.2"
        )
        return UserProfileList(req["memberList"]).UserProfileList

    def get_chat_info(self, chatId: str):
        req = self.getRequest(f"/x{self.comId}/s/chat/thread/{chatId}")
        return Thread(req["thread"]).Thread

    def get_online_users(self, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/live-layer?topic=ndtopic:x{self.comId}:online-members&start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    def get_public_chats(
        self, filterType: str = "recommended", start: int = 0, size: int = 50
    ):
        req = self.getRequest(
            f"/x{self.comId}/s/chat/thread?type=public-all&filterType={filterType}&start={start}&size={size}"
        )
        return ThreadList(req["threadList"]).ThreadList

    def send_message(
        self,
        chatId: str,
        message: str = None,
        messageType: int = 0,
        file: BinaryIO = None,
        fileType: str = None,
        replyTo: str = None,
        mentionUserIds: Union[list, str] = None,
        stickerId: str = None,
        snippetLink: str = None,
        ytVideo: str = None,
        snippetImage: BinaryIO = None,
        embedId: str = None,
        embedType: int = None,
        embedLink: str = None,
        embedTitle: str = None,
        embedContent: str = None,
        embedImage: BinaryIO = None,
    ):

        if message is not None and file is None:
            message = message.replace("[@", "‎‏").replace("@]", "‬‭")

        mentions = []
        if mentionUserIds:
            if isinstance(mentionUserIds, list):
                for mention_uid in mentionUserIds:
                    mentions.append({"uid": mention_uid})
            mentions.append({"uid": mentionUserIds})

        if embedImage:
            if not isinstance(embedImage, str):
                embedImage = [[100, self.upload_image(embedImage, "image"), None]]
            embedImage = [[100, embedImage, None]]

        data = {
            "type": messageType,
            "content": message,
            "attachedObject": {
                "objectId": embedId,
                "objectType": embedType,
                "link": embedLink,
                "title": embedTitle,
                "content": embedContent,
                "mediaList": embedImage,
            },
            "extensions": {"mentionedArray": mentions},
            "clientRefId": int(timestamp() / 10 % 100000000),
            "timestamp": int(timestamp() * 1000),
        }

        if replyTo:
            data["replyMessageId"] = replyTo

        if stickerId:
            data["content"] = None
            data["stickerId"] = stickerId
            data["type"] = 3

        if snippetLink and snippetImage:
            data["attachedObject"] = None
            data["extensions"]["linkSnippetList"] = [
                {
                    "link": snippetLink,
                    "mediaType": 100,
                    "mediaUploadValue": b64encode(snippetImage.read()).decode(),
                    "mediaUploadValueContentType": "image/png",
                }
            ]

        if ytVideo:
            data["content"] = None
            data["mediaType"] = 103
            data["mediaValue"] = ytVideo

        if file:
            data["content"] = None

            if fileType == "audio":
                data["type"] = 2
                data["mediaType"] = 110

            elif fileType == "image":
                data["mediaType"] = 100
                data["mediaUploadValueContentType"] = "image/jpg"
                data["mediaUhqEnabled"] = False

            elif fileType == "gif":
                data["mediaType"] = 100
                data["mediaUploadValueContentType"] = "image/gif"
                data["mediaUhqEnabled"] = False

            else:
                raise TypeError(fileType)

            data["mediaUploadValue"] = b64encode(file.read()).decode()
            data["attachedObject"] = None
            data["extensions"] = None

        req = self.postRequest(f"/x{self.comId}/s/chat/thread/{chatId}/message", data)
        return Json(req)

    def send_web_message(
        self,
        chatId: str,
        message: str = None,
        messageType: int = 0,
        icon: str = None,
        comId: str = None,
    ):
        if comId:
            self.comId = comId

        data = {
            "ndcId": f"x{self.comId}",
            "threadId": chatId,
            "message": {
                "content": message,
                "mediaType": 0,
                "type": messageType,
                "sendFailed": False,
                "clientRefId": 0,
            },
        }

        if icon:
            data["message"]["content"] = None
            data["message"]["uploadId"] = 0
            data["message"]["mediaType"] = 100
            data["message"]["mediaValue"] = icon

        return Json(self.postRequest("/add-chat-message", data, webRequest=True))

    def unfollow(self, userId: str):
        req = self.postRequest(
            f"/x{self.comId}/s/user-profile/{userId}/member/{self.uid}"
        )
        return Json(req)

    def follow(self, userId: Union[str, list]):
        data = {"timestamp": int(timestamp() * 1000)}

        if isinstance(userId, str):
            link = f"/x{self.comId}/s/user-profile/{userId}/member"
        elif isinstance(userId, list):
            link = f"/x{self.comId}/s/user-profile/{self.uid}/joined"
            data["targetUidList"] = userId

        req = self.postRequest(link, data)
        return Json(req)

    def start_chat(
        self,
        userId: Union[str, list],
        title: str = None,
        message: str = None,
        content: str = None,
        chatType: int = 0,
    ):
        if isinstance(userId, str):
            userId = [userId]

        data = {
            "title": title,
            "inviteeUids": userId,
            "initialMessageContent": message,
            "content": content,
            "type": chatType,
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest(f"/x{self.comId}/s/chat/thread", data)
        return Thread(req['thread']).Thread

    def invite_to_chat(self, userId: Union[str, list], chatId: str = None):
        if isinstance(userId, str):
            userId = [userId]

        data = {"uids": userId, "timestamp": int(timestamp() * 1000)}

        req = self.postRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/member/invite", data=data
        )
        return Json(req)

    def edit_profile(
        self,
        nickname: str = None,
        content: str = None,
        icon: BinaryIO = None,
        backgroundColor: str = None,
        backgroundImage: str = None,
        defaultBubbleId: str = None,
    ):
        data = {
            "address": None,
            "latitude": 0,
            "longitude": 0,
            "mediaList": None,
            "eventSource": "UserProfileView",
            "timestamp": int(timestamp() * 1000),
        }

        if content:
            data["content"] = content
        if nickname:
            data["nickname"] = nickname
        if icon:
            data["icon"] = self.upload_media(icon, "image")
        if defaultBubbleId:
            data["extensions"] = {"defaultBubbleId": defaultBubbleId}
        if backgroundColor:
            data["extensions"]["style"]["backgroundColor"] = backgroundColor
        if backgroundImage:
            data["extensions"]["style"]["backgroundMediaList"] = [
                [100, backgroundImage, None, None, None]
            ]

        req = self.postRequest(f"/x{self.comId}/s/user-profile/{self.uid}", data)
        return Json(req)

    def edit_chat(
        self,
        chatId: str,
        title: str = None,
        content: str = None,
        icon: str = None,
        background: str = None,
        keywords: list = None,
        announcement: str = None,
        pinAnnouncement: bool = None,
    ):
        res, data = [], {"timestamp": int(timestamp() * 1000)}

        if title:
            data["title"] = title
        if content:
            data["content"] = content
        if icon:
            data["icon"] = icon
        if keywords:
            data["keywords"] = keywords
        if announcement:
            data["extensions"]["announcement"] = announcement
        if pinAnnouncement:
            data["extensions"]["pinAnnouncement"] = pinAnnouncement
        if background:
            data = {
                "media": [100, background, None],
                "timestamp": int(timestamp() * 1000),
            }
            req = self.postRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.uid}/background",
                data,
            )
            res.append(Json(req))

        req = self.postRequest(f"/x{self.comId}/s/chat/thread/{chatId}", data)
        res.append(Json(req))

        return res

    def chat_settings(
        self,
        chatId: str,
        viewOnly: bool = False,
        doNotDisturb: bool = True,
        canInvite: bool = False,
        canTip: bool = None,
        pin: bool = None,
        coHosts: Union[str, list] = None,
    ):
        res = []

        if doNotDisturb:
            if doNotDisturb:
                opt = 2
            else:
                opt = 1

            data = {"alertOption": opt, "timestamp": int(timestamp() * 1000)}
            req = self.postRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.uid}/alert", data
            )
            res.append(Json(req))

        if viewOnly:
            if viewOnly:
                viewOnly = "enable"
            else:
                viewOnly = "disable"

            req = self.postRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/view-only/{viewOnly}"
            )
            res.append(Json(req))

        if canInvite:
            if canInvite:
                canInvite = "enable"
            else:
                canInvite = "disable"

            req = self.postRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/members-can-invite/{canInvite}"
            )
            res.append(Json(req))

        if canTip:
            if canTip:
                canTip = "enable"
            else:
                canTip = "disable"
            req = self.postRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/tipping-perm-status/{canTip}"
            )
            res.append(Json(req))

        if pin:
            if pin:
                pin = "pin"
            else:
                pin = "unpin"

            req = self.postRequest(f"/x{self.comId}/s/chat/thread/{chatId}/{pin}")
            res.append(Json(req))

        if coHosts:
            data = {"uidList": coHosts, "timestamp": int(timestamp() * 1000)}
            req = self.postRequest(f"{self.comId}/s/chat/thread/{chatId}/co-host", data)
            res.append(Json(req))

        return res

    def like_blog(self, blogId: str = None, wikiId: str = None):
        data = {"value": 4, "timestamp": int(timestamp() * 1000)}

        if blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/vote?cv=1.2&value=4"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/vote?cv=1.2&value=4"
        else:
            raise TypeError("Please put wiki or blog Id")

        req = self.postRequest(link, data)
        return Json(req)

    def unlike_blog(self, blogId: str = None, wikiId: str = None):
        if blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/vote?eventSource=FeedList"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/vote?eventSource=FeedList"
        else:
            raise TypeError("Please put wikiId or blogId")

        req = self.deleteRequest(link)
        return Json(req)

    def change_titles(self, userId: str, titles: list, colors: list):
        t = []
        for title, color in zip(titles, colors):
            t.append({"title": title, "color": color})
        data = {
            "adminOpName": 207,
            "adminOpValue": {"titles": t},
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest(f"/x{self.comId}/s/user-profile/{userId}/admin", data)
        return Json(req)

    def like_comment(
        self, commentId: str, blogId: str = None, wikiId: str = None, userId: str = None
    ):
        data = {"value": 1}

        if blogId:
            data["eventSource"] = "PostDetailView"
            link = f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?cv=1.2&value=1"
        elif wikiId:
            data["eventSource"] = "PostDetailView"
            link = f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/vote?cv=1.2&value=1"
        elif userId:
            data["eventSource"] = "UserProfileView"
            link = f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/vote?cv=1.2&value=1"
        else:
            raise TypeError("Please put a wiki or user or blog Id")

        req = self.postRequest(link, data)
        return Json(req)

    def unlike_comment(
        self, commentId: str, blogId: str = None, wikiId: str = None, userId: str = None
    ):
        if blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/g-vote?eventSource=PostDetailView"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/g-vote?eventSource=PostDetailView"
        elif userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/g-vote?eventSource=UserProfileView"
        else:
            raise TypeError("Please put a wiki or user or blog Id")

        req = self.deleteRequest(link)
        return Json(req)

    def comment(
        self,
        comment: str,
        userId: str = None,
        blogId: str = None,
        wikiId: str = None,
        replyTo: str = None,
        isGuest: bool = False,
    ):
        data = {"content": comment, "timestamp": int(timestamp() * 1000)}

        if replyTo:
            data["respondTo"] = replyTo
        if isGuest:
            comType = "g-comment"
        else:
            comType = "comment"

        if userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/{comType}"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/{comType}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/{comType}"
        else:
            raise TypeError("Please put a wiki or user or blog Id")

        req = self.postRequest(link, data)
        return Json(req)

    def delete_comment(
        self, commentId: str, userId: str = None, blogId: str = None, wikiId: str = None
    ):
        if userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}"
        else:
            raise TypeError("Please put blog or wiki or user Id")

        req = self.deleteRequest(link)
        return Json(req)

    def edit_comment(
        self,
        commentId: str,
        comment: str,
        userId: str = None,
        blogId: str = None,
        wikiId: str = None,
        replyTo: str = None,
        isGuest: bool = False,
    ):
        data = {"content": comment, "timestamp": int(timestamp() * 1000)}
        if replyTo:
            data["respondTo"] = replyTo
        if isGuest:
            comType = "g-comment"
        else:
            comType = "comment"

        if userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/{comType}/{commentId}"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/{comType}/{commentId}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/{comType}/{commentId}"
        else:
            raise TypeError("Please put blog or wiki or user Id")

        req = self.postRequest(link, data)
        return Comment(req).Comments

    def get_comment_info(
        self,
        commentId: str,
        userId: str = None,
        blogId: str = None,
        wikiId: str = None,
        isGuest: bool = False,
    ):
        if isGuest:
            comType = "g-comment"
        else:
            comType = "comment"

        if userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/{comType}/{commentId}"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/{comType}/{commentId}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/{comType}/{commentId}"
        else:
            raise TypeError("Please put blog or wiki or user Id")

        req = self.getRequest(link)
        return Comment(req).Comments

    def get_wall_comments(
        self, userId: str, sorting: str = "newest", start: int = 0, size: int = 25
    ):
        sorting = sorting.lower().replace("top", "vote")
        if sorting not in ["newest", "oldest", "vote"]:
            raise TypeError("حط تايب يا حمار")

        req = self.getRequest(
            f"/x{self.comId}/s/user-profile/{userId}/comment?sort={sorting}&start={start}&size={size}"
        )
        return CommentList(req["commentList"]).CommentList

    def get_blog_comments(
        self,
        wikiId: str = None,
        blogId: str = None,
        quizId: str = None,
        sorting: str = "newest",
        size: int = 25,
        start: int = 0,
    ):
        sorting = sorting.lower().replace("top", "vote")
        if sorting not in ["newest", "oldest", "vote"]:
            raise TypeError("حط تايب يا حمار")

        if quizId:
            blogId = quizId
        if blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/comment?sort={sorting}&start={start}&size={size}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/comment?sort={sorting}&start={start}&size={size}"
        else:
            raise TypeError("Please choose a wiki or a blog")

        req = self.getRequest(link)
        return CommentList(req["commentList"]).CommentList

    def vote_comment(self, blogId: str, commentId: str, value: bool = True):
        if value:
            value = 1
        else:
            value = -1

        data = {"value": value, "timestamp": int(timestamp() * 1000)}
        req = self.postRequest(
            f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?cv=1.2&value=1",
            data,
        )
        return Json(req)

    def vote_poll(self, blogId: str, optionId: str):
        data = {"value": 1, "timestamp": int(timestamp() * 1000)}
        req = self.postRequest(
            f"/x{self.comId}/s/blog/{blogId}/poll/option/{optionId}/vote", data
        )
        return Json(req)

    def get_blog_info(
        self, blogId: str = None, wikiId: str = None, folderId: str = None
    ):
        if blogId:
            link = f"/x{self.comId}/s/blog/{blogId}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}"
        elif folderId:
            link = f"/x{self.comId}/s/shared-folder/files/{folderId}"
        else:
            raise TypeError("Please put a wiki or blog Id")

        req = self.getRequest(link)
        return GetInfo(req).GetInfo

    def get_blogs(self, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/feed/featured?start={start}&size={size}"
        )
        return BlogList(req["featuredList"]).BlogList

    def get_blogs_more(self, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/feed/featured-more?start={start}&size={size}"
        )
        return BlogList(req["blogList"]).BlogList

    def get_blogs_all(self, start: int = 0, size: int = 25, pagingType: str = "t"):
        req = self.getRequest(
            f"/x{self.comId}/s/feed/blog-all?pagingType={pagingType}&start={start}&size={size}"
        )
        return RecentBlogs(req["blogList"]).RecentBlogs

    def tip_coins(
        self,
        coins: int,
        chatId: str = None,
        blogId: str = None,
        wikiId: str = None,
        transactionId: str = None,
    ):
        if transactionId is None:
            transactionId = str(UUID(hexlify(os.urandom(16)).decode("ascii")))
        data = {
            "coins": int(coins),
            "tippingContext": {"transactionId": transactionId},
            "timestamp": int(timestamp() * 1000),
        }

        if chatId:
            link = f"/x{self.comId}/s/chat/thread/{chatId}/tipping"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/tipping"
        elif wikiId:
            link = f"/x{self.comId}/s/tipping"
            data["objectType"] = 2
            data["objectId"] = wikiId
        else:
            raise TypeError("Please put a wiki or chat or blog Id")

        req = self.postRequest(link, data)
        return Json(req)

    def check_in(self, timezone: int = 180):
        data = {"timezone": timezone, "timestamp": int(timestamp() * 1000)}
        req = self.postRequest(f"/x{self.comId}/s/check-in", data)
        return Json(req)

    def check_in_lottery(self, timezone: int = 180):
        data = {"timezone": timezone, "timestamp": int(timestamp() * 1000)}
        req = self.postRequest(f"/x{self.comId}/s/check-in/lottery", data)
        return Json(req)

    def delete_message(
        self, chatId: str, messageId: str, asStaff: bool = False, reason: str = None
    ):
        data = {"adminOpName": 102, "timestamp": int(timestamp() * 1000)}

        if asStaff and reason:
            data["adminOpNote"] = {"content": reason}
            req = self.postRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/message/{messageId}/admin", data
            )
        else:
            req = self.deleteRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/message/{messageId}"
            )

        return Json(req)

    def invite_by_host(self, chatId: str, userId: Union[str, list]):
        req = self.postRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/member/{userId}/invite-av-chat"
        )
        return Json(req)

    def strike(self, userId: str, time: str, title: str = None, reason: str = None):
        times = {
            "1-Hours": 3600,
            "3-Hours": 10800,
            "6-Hours": 21600,
            "12-Hours": 43200,
            "24-Hours": 86400,
        }.get(time, 3600)

        data = {
            "uid": userId,
            "title": title,
            "content": reason,
            "attachedObject": {"objectId": userId, "objectType": 0},
            "penaltyType": 1,
            "penaltyValue": times,
            "adminOpNote": {},
            "noticeType": 4,
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest(f"/x{self.comId}/s/notice", data)
        return Json(req)

    def ban(self, userId: str, reason: str, banType: int = None):
        data = {
            "reasonType": banType,
            "note": {"content": reason},
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest(f"/x{self.comId}/s/user-profile/{userId}/ban", data)
        return Json(req)

    def unban(self, userId: str, reason: str = "هذا العضو كان شاطر اخر كم يوم"):
        data = {"note": {"content": reason}, "timestamp": int(timestamp() * 1000)}
        req = self.postRequest(f"/x{self.comId}/s/user-profile/{userId}/unban", data)
        return Json(req)

    def hide(
        self,
        note: str = None,
        blogId: str = None,
        userId: str = None,
        wikiId: str = None,
        chatId: str = None,
    ):
        data = {
            "adminOpName": 18 if userId else 110,
            "adminOpValue": None if userId else 9,
            "timestamp": int(timestamp() * 1000),
        }

        if userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/admin"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/admin"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/admin"
        elif chatId:
            link = f"/x{self.comId}/s/chat/thread/{chatId}/admin"
        else:
            raise TypeError("Please put a wiki or user or chat or blog Id")
        if note:
            data["adminOpNote"] = {"content": note}

        req = self.postRequest(link, data)
        return Json(req)

    def unhide(
        self,
        note: str = None,
        blogId: str = None,
        userId: str = None,
        wikiId: str = None,
        chatId: str = None,
    ):
        data = {
            "adminOpName": 19 if userId else 110,
            "adminOpValue": 0,
            "timestamp": int(timestamp() * 1000),
        }

        if userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/admin"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/admin"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/admin"
        elif chatId:
            link = f"/x{self.comId}/s/chat/thread/{chatId}/admin"
        else:
            raise TypeError("Please put a wiki or user or chat or blog Id")
        if note:
            data["adminOpNote"] = {"content": note}

        req = self.postRequest(link, data)
        return Json(req)

    def send_warning(self, userId: str, reason: str = None):
        data = {
            "uid": userId,
            "title": "Custom",
            "content": reason,
            "attachedObject": {"objectId": userId, "objectType": 0},
            "penaltyType": 0,
            "adminOpNote": {},
            "noticeType": 7,
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest(f"/x{self.comId}/s/notice", data)
        return Json(req)

    def invite_to_voice_chat(self, userId: str = None, chatId: str = None):
        data = {"uid": userId, "timestamp": int(timestamp() * 1000)}
        req = self.postRequest(
            f"/g/x{self.comId}/chat/thread/{chatId}/vvchat-presenter/invite", data
        )
        return Json(req)

    def post_blog(self, title: str, content: str, fansOnly: bool = False):
        data = {
            "extensions": {"fansOnly": fansOnly},
            "content": content,
            "latitude": 0,
            "longitude": 0,
            "title": title,
            "type": 0,
            "contentLanguage": "ar",
            "eventSource": "GlobalComposeMenu",
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest(f"/x{self.comId}/s/blog", data)
        return Json(req)

    def post_wiki(
        self,
        title: str,
        content: str,
        fansOnly: bool = False,
        icon: str = None,
        backgroundColor: str = None,
        keywords: Union[str, list] = None,
    ):
        data = {
            "extensions": {
                "fansOnly": fansOnly,
                "props": [],
                "style": {"backgroundColor": backgroundColor},
            },
            "content": content,
            "keywords": keywords,
            "label": title,
            "latitude": 0,
            "longitude": 0,
            "eventSource": "UserProfileView",
            "timestamp": int(timestamp() * 1000),
        }
        if icon:
            data["icon"] = icon

        req = self.postRequest(f"/x{self.comId}/s/item", data)
        return Json(req)

    def delete_blog(self, blogId: str):
        req = self.deleteRequest(f"/x{self.comId}/s/blog/{blogId}")
        return Json(req)

    def delete_wiki(self, wikiId: str):
        req = self.deleteRequest(f"/x{self.comId}/s/item/{wikiId}")
        return Json(req)

    def activate_status(self, status: int = 1):
        data = {
            "onlineStatus": status,
            "duration": 86400,
            "timestamp": int(timestamp() * 1000),
        }
        req = self.postRequest(
            f"/x{self.comId}/s/user-profile/{self.uid}/online-status", data
        )
        return Json(req)

    def subscribe(self, userId: str, autoRenew: str = False, transactionId: str = None):
        if transactionId is None:
            transactionId = str(UUID(hexlify(os.urandom(16)).decode("ascii")))
        data = {
            "paymentContext": {
                "transactionId": transactionId,
                "isAutoRenew": autoRenew,
            },
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest(f"/x{self.comId}/s/influencer/{userId}/subscribe", data)
        return Json(req)

    def submit_wiki(self, wikiId: str, message: str = None):
        data = {
            "message": message,
            "itemId": wikiId,
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest(f"/x{self.comId}/s/knowledge-base-request", data)
        return Json(req)

    def edit_blog(
        self,
        title: str,
        content: str,
        blogId: str = None,
        wikiId: str = None,
        fansOnly: bool = False,
        backgroundColor: str = None,
        media: list = None,
    ):
        data = {
            "title": title,
            "content": content,
            "timestamp": int(timestamp() * 1000),
        }

        if media:
            data["mediaList"] = [[100, media, None, "XYZ", None, {"fileName": "Amino"}]]
        if fansOnly:
            data["extensions"]["fansOnly"] = True
        if backgroundColor:
            data["extensions"] = {"backgroundColor": backgroundColor}

        if blogId:
            link = f"/x{self.comId}/s/blog/{blogId}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}"
        else:
            raise TypeError("Please put blogId or wikiId")

        req = self.postRequest(link, data)
        return Json(req)

    def get_chat_bubbles(self, start: int = 0, size: int = 20):
        req = self.getRequest(
            f"/x{self.comId}/s/chat/chat-bubble?type=all-my-bubbles&start={start}&size={size}"
        )
        return BubbleList(req["chatBubbleList"]).BubbleList

    def select_bubble(self, bubbleId: str, apply: int = 0, chatId: str = None):
        data = {
            "bubbleId": bubbleId,
            "applyToAll": apply,
            "timestamp": int(timestamp() * 1000),
        }
        if chatId:
            data["threadId"] = chatId

        req = self.postRequest(f"/x{self.comId}/s/chat/thread/apply-bubble")
        return Json(req)

    def delete_chat_bubble(self, bubbleId: str):
        req = self.deleteRequest(url=f"/x{self.comId}/s/chat/chat-bubble/{bubbleId}")
        return Json(req)

    def get_chat_bubble_templates(self, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/chat/chat-bubble/templates?start={start}&size={size}"
        )
        return BubbleTemplates(req["templateList"])

    def upload_custom_bubble(self, templateId: str):
        req = self.postRequest(
            f"/x{self.comId}/s/chat/chat-bubble/templates/{templateId}/generate"
        )
        return Json(req)

    def kick(self, chatId: str, userId: str, rejoin: bool = True):
        if rejoin:
            rejoin = 1
        if not rejoin:
            rejoin = 0

        req = self.deleteRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/member/{userId}?allowRejoin={rejoin}"
        )
        return Json(req)

    def block(self, userId: str):
        req = self.postRequest(f"/x{self.comId}/s/block/{userId}")
        return Json(req)

    def flag(
        self,
        reason: str,
        flagType: str = "spam",
        userId: str = None,
        wikiId: str = None,
        blogId: str = None,
    ):
        types = {
            "violence": 106,
            "hate": 107,
            "suicide": 108,
            "troll": 109,
            "nudity": 110,
            "bully": 0,
            "off-topic": 4,
            "spam": 2,
        }

        data = {"message": reason, "timestamp": int(timestamp() * 1000), "flagType": types.get(flagType, 106)}

        if userId:
            data["objectId"] = userId
            data["objectType"] = 0
        elif blogId:
            data["objectId"] = blogId
            data["objectType"] = 1
        elif wikiId:
            data["objectId"] = wikiId
            data["objectType"] = 2
        else:
            raise TypeError("choose a certain type to report")

        req = self.postRequest(f"/x{self.comId}/s/flag", data)
        return Json(req)
#thanks to V¡ktor#9475 for ideas
    def send_active_time(self, tz: int = int(-timezones // 1000), timers: list = None):
        data = {
            "userActiveTimeChunkList": timers if timers else [{"start": int(timestamp()), "end": int(timestamp() + 300)}],
            "timestamp": int(timestamp() * 1000),
            "optInAdsFlags": 2147483647,
            "timezone": tz,
        }

        req = self.postRequest(
            f"/x{self.comId}/s/community/stats/user-active-time", data, minify=True
        )
        return Json(req)

    def transfer_host(self, chatId: str, userIds: list):
        data = {"uidList": userIds, "timestamp": int(timestamp() * 1000)}
        req = self.postRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/transfer-organizer", data
        )
        return Json(req)

    def accept_host(self, chatId: str, requestId: str):
        data = {"timestamp": int(timestamp() * 1000)}
        req = self.postRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/transfer-organizer/{requestId}/accept",
            data,
        )
        return Json(req)

    def set_cohost(self, chatId : str, asistent_id : [str, list]):
         data = {
            "uidList": asistent_id,
            "timestamp": int(timestamp() * 1000)
        }

         req = self.postRequest(f"/x{self.comId}/s/chat/thread/{chatId}/co-host", data)
         return Json(req)


    def del_cohost(self, chatId : str, userId : str):
           req = self.deleteRequest(f"/x{self.comId}/s/chat/thread/{chatId}/co-host/{userId}")
           return Json(req)

    def get_quizzes(self, quizzesType: str = "recent", start: int = 0, size: int = 25):
        link = {
            "recent": f"x{self.comId}/s/blog?type=quizzes-recent&start={start}&size={size}",
            "trending": f"x{self.comId}/s/feed/quiz-trending?start={start}&size={size}",
            "best": f"x{self.comId}/s/feed/quiz-best-quizzes?start={start}&size={size}",
        }.get(quizzesType)

        req = self.getRequest(link)
        return BlogList(req["blogList"]).BlogList

    def get_quiz_questions(self, quizId: str):
        req = self.getRequest(f"/x{self.comId}/s/blog/{quizId}?action=review")
        return QuizQuestionList(req["blog"]["quizQuestionList"]).QuizQuestionList

    def play_quiz(self, quizId: str, questions: list, answers: list, mode: int = 0):
        data = {
            "mode": mode,
            "quizAnswerList": [
                {"optIdList": [answer], "quizQuestionId": question, "timeSpent": 0.0}
                for answer, question in zip(answers, questions)
            ],
            "timestamp": int(timestamp() * 1000),
        }
        req = self.postRequest(f"/x{self.comId}/s/blog/{quizId}/quiz/result", data)
        return req

    def get_quiz_rankings(self, quizId: str, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/blog/{quizId}/quiz/result?start={start}&size={size}"
        )
        return QuizRankings(req).QuizRankings

    def search_user(self, username: str, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/user-profile?type=name&q={username}&start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    def search_blog(self, words: str, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/blog?type=keywords&q={words}&start={start}&size={size}"
        )
        return BlogList(req["blogList"]).BlogList

    def get_notifications(self, size: int = 25, pagingType: str = "t"):
        req = self.getRequest(
            f"/x{self.comId}/s/notification?pagingType={pagingType}&size={size}"
        )
        return NotificationList(req).NotificationList

    def get_notices(
        self, start: int = 0, size: int = 25, noticeType: str = "usersV2", status: int = 1
    ):
        req = self.getRequest(
            f"/x{self.comId}/s/notice?type={noticeType}&status={status}&start={start}&size={size}"
        )
        return NoticeList(req).NoticeList

    def accept_promotion(self, requestId: str):
        if not isinstance(requestId, str):
            raise TypeError(f"Please use a string not {type(requestId)}")
        req = self.postRequest(f"/x{self.comId}/s/notice/{requestId}/accept")
        return Json(req)

    def decline_promotion(self, requestId: str):
        if not isinstance(requestId, str):
            raise TypeError(f"Please use a string not {type(requestId)}")
        req = self.postRequest(f"/x{self.comId}/s/notice/{requestId}/decline")
        return Json(req)

    def sendWebActive(self):
        data = {"ndcId": self.comId}
        return self.postRequest(
            "/community/stats/web-user-active-time", data, webRequest=True
        )

    def get_recent_blogs(self, pageToken: str = None, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/x{self.comId}/s/feed/blog-all?pagingType=t&start={start}&size={size}"
        )
        return RecentBlogs(req["BlogList"]).RecentBlogs

    def publish_to_featured(
        self,
        time: int,
        userId: str = None,
        chatId: str = None,
        blogId: str = None,
        wikiId: str = None,
    ):
        time = (
            {1: 3600, 2: 7200, 3: 10800}.get(time)
            if chatId
            else {1: 86400, 2: 172800, 3: 259200}.get(time)
        )

        data = {
            "adminOpName": 114,
            "adminOpValue": {"featuredDuration": time},
            "timestamp": int(timestamp() * 1000),
        }

        if userId:
            featuredType, endpoint = 4, f"user-profile/{userId}"
        elif blogId:
            featuredType, endpoint = 1, f"blog/{blogId}"
        elif wikiId:
            featuredType, endpoint = 1, f"item/{blogId}"
        elif chatId:
            featuredType, endpoint = 5, f"chat/thread/{chatId}"
        else:
            raise TypeError("Please Specify Object ID")

        data["adminOpValue"]["featuredType"] = featuredType
        req = self.postRequest(f"/x{self.comId}/s/{endpoint}/admin", data)
        return Json(req)

    def remove_from_featured(
        self,
        userId: str = None,
        chatId: str = None,
        blogId: str = None,
        wikiId: str = None,
    ):
        data = {
            "adminOpName": 114,
            "adminOpValue": {"featuredType": 0},
            "timestamp": int(timestamp() * 1000),
        }

        if userId:
            endpoint = f"user-profile/{userId}"
        elif blogId:
            endpoint = f"blog/{blogId}"
        elif wikiId:
            endpoint = f"item/{blogId}"
        elif chatId:
            endpoint = f"chat/thread/{chatId}"
        else:
            raise TypeError("Please Specify Object ID")

        req = self.postRequest(f"/x{self.comId}/s/{endpoint}/admin", data)
        return Json(req)



from easy_events import Events, Parameters

from .local import SubClient


class Bot(Events):
    def __init__(self, prefix: str = "!"):
        Events.__init__(self, prefix=prefix, str_only=True, first_parameter_object=True, default_event=False)

        # to make @client.command()
        self.command = self.add_event

    def build_parameters(self, obj_data):
        data = Parameters(data=obj_data.message.content, prefix=self.prefix, str_only=True)

        for k, v in obj_data.__dict__.items():
            setattr(data, k, v)

        for k, v in obj_data.message.__dict__.items():
            if k != "json":
                setattr(data, k, v)

        data.subClient = data.local = SubClient(comId=data.comId, acm=True)

        data.authorId = data.message.author.userId
        data.authorName = data.message.author.nickname
        data.messageId = data.message.messageId
        data.authorIcon = data.message.author.icon

        try: data.level = data.message.json["author"]["level"]
        except: pass
        try: data.reputation = data.message.json["author"]["reputation"]
        except: pass

        data.replySrc = None
        data.replyId = None

        if data.message.extensions and data.message.extensions.get('replyMessage') and data.message.extensions['replyMessage'].get('mediaValue'):
            data.replySrc = data.message.extensions['replyMessage']['mediaValue'].replace('_00.', '_hq.')
            data.replyId = data.message.extensions['replyMessage']['messageId']

        data.replyMsg = None

        if data.message.extensions and data.message.extensions.get('replyMessage') and data.message.extensions['replyMessage'].get('content'):
            data.replyMsg = data.message.extensions['replyMessage']['content']
            data.replyId = data.message.extensions['replyMessage']['messageId']

        return data



from setuptools import setup, find_packages
from k_amino import __version__

with open("README.md", "r") as stream:
    README = stream.read()


setup(
    name="k-amino.py",
    version=__version__,
    url="https://github.com/kwel999/KAmino",
    download_url="https://github.com/kwel999/KAmino/archive/refs/heads/main.zip",
    description="Amino Bots with python!",
    long_description=README,
    long_description_content_type="text/markdown",
    author="KWEL",
    author_email="itskwel999@gmail.com",
    license="Apache",
    keywords=[
        "api",
        "python",
        "python3",
        "python3.x",
        "KWEL",
        "kwel999",
        "kwel.py",
        "Amino",
        "kamino",
        "kamino py"
        "K-Amino",
        "kamino",
        "kamino",
        "kamino-bot",
        "kamino-bots",
        "kamino-bot",
        "ndc",
        "narvii.apps",
        "aminoapps",
        "kamino-py",
        "kamino",
        "kamino-bot",
        "narvii",
    ],
    include_package_data=True,
    install_requires=[
        "JSON_minify",
        "httpx",
        "setuptools",
        "aiohttp",
        "websocket-client==1.3.1",
        "websockets",
        "ujson",
        "requests",
        "easy-events==2.8.2"
    ],
    setup_requires=["wheel"],
    packages=find_packages(),
)



import os
from base64 import b64encode
from binascii import hexlify
from time import time as timestamp
from typing import BinaryIO, Union
from uuid import UUID

from ..lib.objects import *
from ..lib.sessions import Session
from .sockets import Wss


class Client(Wss, Session):
    def __init__(self, deviceId: str = None, proxies: dict = None, trace: bool = False, bot: bool = False):
        self.trace = trace
        self.proxies = proxies
        self.deviceId = deviceId

        Wss.__init__(self, self, trace=self.trace, is_bot=bot)
        Session.__init__(self, proxies=self.proxies, staticDevice=self.deviceId)

    def change_lang(self, lang: str = "en"):
        self.updateHeaders(lang=lang)

    def sid_login(self, sid: str, socket = False):
        finalSessionId = sid if "sid=" in sid else f"sid={sid}"

        info = self.get_account_info()
        self.settings(
            user_userId=info.userId,
            user_session=finalSessionId,
        )

        if socket: self.launch()
        return info

    def login(
        self,
        email: str = None,
        password: str = None,
        secret: str = None,
        socket: bool = False,
    ):

        if not ((email and password) or secret):
            raise ValueError("Please provide VALID login info")
        data = {
            "email": email if email else "",
            "secret": f"0 {password}" if password else secret,
            "clientType": 100,
            "action": "normal",
            "deviceID": self.deviceId,
            "v": 2,
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest("/g/s/auth/login", data)
        self.settings(
            user_userId=req["auid"],
            user_session=f'sid={req["sid"]}',
            user_secret=secret if secret else req["secret"],
        )
        if socket or self.is_bot:
            self.launch()
        return Login(req)

    def logout(self):
        data = {
            "deviceID": self.deviceId,
            "clientType": 100,
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest("/g/s/auth/logout", data)
        self.settings()  # Resets the sid and uid and secret to None in sessions.py

        if self.isOpened: self.close()
        return Json(req)

    def check_device(self, deviceId: str) -> str:
        data = {
            "deviceID": deviceId,
            "timestamp": int(timestamp() * 1000),
            "clientType": 100,
        }
        self.headers["NDCDEVICEID"] = deviceId
        req = self.postRequest("/g/s/device/", data)
        return Json(req)

    def upload_image(self, image: BinaryIO):
        newHeaders = {"content-type": "image/jpg", "content-length": str(len(image.read()))}
        return self.postRequest("/g/s/media/upload", data=image, newHeaders=newHeaders)["mediaValue"]

    def send_verify_code(self, email: str):
        data = {
            "identity": email,
            "type": 1,
            "deviceID": self.deviceId,
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest("/g/s/auth/request-security-validation", data)
        return Json(req)

    def accept_host(self, requestId: str, chatId: str):
        req = self.postRequest(
            f"/g/s/chat/thread/{chatId}/transfer-organizer/{requestId}/accept"
        )
        return Json(req)

    def verify_account(self, email: str, code: str):
        data = {
            "type": 1,
            "identity": email,
            "data": {"code": code},
            "deviceID": self.deviceId,
        }
        req = self.postRequest("/g/s/auth/activate-email", data)
        return Json(req)

    def restore(self, email: str, password: str):
        data = {
            "secret": f"0 {password}",
            "deviceID": self.deviceId,
            "email": email,
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest("/g/s/account/delete-request/cancel", data)
        return Json(req)

    def delete_account(self, password: str = None):
        data = {
            "deviceID": self.deviceId,
            "secret": f"0 {password}",
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest("/g/s/account/delete-request", data)
        return Json(req)

    def get_account_info(self):
        req = self.getRequest("/g/s/account")
        return Account(req["account"])

    def claim_coupon(self):
        req = self.postRequest("/g/s/coupon/new-user-coupon/claim")
        return Json(req)

    def change_amino_id(self, aminoId: str = None):
        data = {"aminoId": aminoId, "timestamp": int(timestamp() * 1000)}
        req = self.postRequest("/g/s/account/change-amino-id", data)
        return Json(req)

    def get_my_communities(self, start: int = 0, size: int = 25):
        req = self.getRequest(f"/g/s/community/joined?v=1&start={start}&size={size}")
        return CommunityList(req["communityList"]).CommunityList

    def get_chat_threads(self, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/g/s/chat/thread?type=joined-me&start={start}&size={size}"
        )
        return ThreadList(req["threadList"]).ThreadList

    def get_chat_info(self, chatId: str):
        req = self.getRequest(f"/g/s/chat/thread/{chatId}")
        return Thread(req["thread"]).Thread

    def leave_chat(self, chatId: str):
        req = self.deleteRequest(f"/g/s/chat/thread/{chatId}/member/{self.uid}")
        return Json(req)

    def join_chat(self, chatId: str):
        req = self.postRequest(f"/g/s/chat/thread/{chatId}/member/{self.uid}")
        return Json(req)

    def start_chat(
        self,
        userId: Union[str, list],
        title: str = None,
        message: str = None,
        content: str = None,
        chatType: int = 0,
    ):

        if isinstance(userId, str):
            userId = [userId]

        data = {
            "title": title,
            "inviteeUids": userId,
            "initialMessageContent": message,
            "content": content,
            "type": chatType,
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest(f"/g/s/chat/thread", data)
        return Thread(req['thread']).Thread

    def get_from_link(self, link: str):
        req = self.getRequest(f"/g/s/link-resolution?q={link}")
        return FromCode(req["linkInfoV2"]["extensions"]).FromCode

    def edit_profile(
        self,
        nickname: str = None,
        content: str = None,
        icon: BinaryIO = None,
        backgroundColor: str = None,
        backgroundImage: str = None,
        defaultBubbleId: str = None,
    ):
        data = {
            "address": None,
            "latitude": 0,
            "longitude": 0,
            "mediaList": None,
            "eventSource": "UserProfileView",
            "timestamp": int(timestamp() * 1000),
        }

        if content:
            data["content"] = content
        if nickname:
            data["nickname"] = nickname
        if icon:
            data["icon"] = self.upload_image(icon)
        if backgroundColor:
            data["extensions"]["style"]["backgroundColor"] = backgroundColor
        if defaultBubbleId:
            data["extensions"] = {"defaultBubbleId": defaultBubbleId}
        if backgroundImage:
            data["extensions"]["style"] = {
                "backgroundMediaList": [[100, backgroundImage, None, None, None]]
            }

        req = self.postRequest(f"/g/s/user-profile/{self.uid}", data)
        return Json(req)

    def flag_community(self, comId: str, reason: str, flagType: int):
        data = {
            "objectId": comId,
            "objectType": 16,
            "flagType": flagType,
            "message": reason,
            "timestamp": int(timestamp() * 1000),
        }
        req = self.postRequest(f"/x{comId}/s/g-flag", data)
        return Json(req)

    def leave_community(self, comId: str):
        req = self.postRequest(f"x/{comId}/s/community/leave")
        return Json(req)

    def join_community(self, comId: str, InviteId: str = None):
        data = {"timestamp": int(timestamp() * 1000)}
        if InviteId:
            data["invitationId"] = InviteId

        req = self.postRequest(f"/x{comId}/s/community/join", data)
        return Json(req)

    def flag(
        self,
        reason: str,
        flagType: str = "spam",
        userId: str = None,
        wikiId: str = None,
        blogId: str = None,
    ):
        types = {
            "violence": 106,
            "hate": 107,
            "suicide": 108,
            "troll": 109,
            "nudity": 110,
            "bully": 0,
            "off-topic": 4,
            "spam": 2,
        }

        data = {"message": reason, "timestamp": int(timestamp() * 1000)}

        if flagType in types:
            data["flagType"] = types.get(flagType)
        else:
            raise TypeError("choose a certain type to report")

        if userId:
            data["objectId"] = userId
            data["objectType"] = 0
        elif blogId:
            data["objectId"] = blogId
            data["objectType"] = 1
        elif wikiId:
            data["objectId"] = wikiId
            data["objectType"] = 2
        else:
            raise TypeError("Please put blog or user or wiki Id")

        req = self.postRequest("/g/s/flag", data)
        return Json(req)

    def unfollow(self, userId: str):
        req = self.postRequest(f"/g/s/user-profile/{userId}/member/{self.uid}")
        return Json(req)

    def follow(self, userId: Union[str, list]):
        data = {"timestamp": int(timestamp() * 1000)}

        if isinstance(userId, str):
            link = f"/g/s/user-profile/{userId}/member"
        elif isinstance(userId, list):
            data["targetUidList"] = userId
            link = f"/g/s/user-profile/{self.uid}/joined"

        req = self.postRequest(link, data)
        return Json(req)

    def get_member_following(self, userId: str, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/g/s/user-profile/{userId}/joined?start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    def get_member_followers(self, userId: str, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/g/s/user-profile/{userId}/member?start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    def get_member_visitors(self, userId: str, start: int = 0, size: int = 25):
        req = self.getRequest(
            f"/g/s/user-profile/{userId}/visitors?start={start}&size={size}"
        )
        return VisitorsList(req["visitors"]).VisitorsList

    def get_blocker_users(self, start: int = 0, size: int = 25):
        req = self.getRequest(f"/g/s/block/full-list?start={start}&size={size}")
        return req["blockerUidList"]

    def get_blocked_users(self, start: int = 0, size: int = 25):
        req = self.getRequest(f"/g/s/block/full-list?start={start}&size={size}")
        return req["blockedUidList"]

    def get_wall_comments(
        self, userId: str, sorting: str = "newest", start: int = 0, size: int = 25
    ):
        sorting = sorting.lower().replace("top", "vote")
        if sorting not in ["newest", "oldest", "vote"]:
            raise TypeError("حط تايب يا حمار")

        req = self.getRequest(
            f"/g/s/user-profile/{userId}/g-comment?sort={sorting}&start={start}&size={size}"
        )
        return CommentList(req["commentList"]).CommentList

    def get_blog_comments(
        self,
        wikiId: str = None,
        blogId: str = None,
        sorting: str = "newest",
        size: int = 25,
        start: int = 0,
    ):
        sorting = sorting.lower().replace("top", "vote")

        if sorting not in ["newest", "oldest", "vote"]:
            raise TypeError("Please insert a valid sorting")
        if blogId:
            link = (
                f"/g/s/blog/{blogId}/comment?sort={sorting}&start={start}&size={size}"
            )
        elif wikiId:
            link = (
                f"/g/s/item/{wikiId}/comment?sort={sorting}&start={start}&size={size}"
            )
        else:
            raise TypeError("Please choose a wiki or a blog")

        req = self.getRequest(link)
        return CommentList(req["commentList"]).CommentList

    def send_message(
        self,
        chatId: str,
        message: str = None,
        messageType: int = 0,
        file: BinaryIO = None,
        fileType: str = None,
        replyTo: str = None,
        mentionUserIds: Union[list, str] = None,
        stickerId: str = None,
        snippetLink: str = None,
        ytVideo: str = None,
        snippetImage: BinaryIO = None,
        embedId: str = None,
        embedType: int = None,
        embedLink: str = None,
        embedTitle: str = None,
        embedContent: str = None,
        embedImage: BinaryIO = None,
    ):

        if message is not None and file is None:
            message = message.replace("[@", "‎‏").replace("@]", "‬‭")

        mentions = []
        if mentionUserIds:
            if isinstance(mentionUserIds, list):
                for mention_uid in mentionUserIds:
                    mentions.append({"uid": mention_uid})
            mentions.append({"uid": mentionUserIds})

        if embedImage:
            if not isinstance(embedImage, str):
                embedImage = [[100, self.upload_image(embedImage), None]]
            embedImage = [[100, embedImage, None]]

        data = {
            "type": messageType,
            "content": message,
            "attachedObject": {
                "objectId": embedId,
                "objectType": embedType,
                "link": embedLink,
                "title": embedTitle,
                "content": embedContent,
                "mediaList": embedImage,
            },
            "extensions": {"mentionedArray": mentions},
            "clientRefId": int(timestamp() / 10 % 100000000),
            "timestamp": int(timestamp() * 1000),
        }

        if replyTo:
            data["replyMessageId"] = replyTo

        if stickerId:
            data["content"] = None
            data["stickerId"] = stickerId
            data["type"] = 3

        if snippetLink and snippetImage:
            data["attachedObject"] = None
            data["extensions"]["linkSnippetList"] = [
                {
                    "link": snippetLink,
                    "mediaType": 100,
                    "mediaUploadValue": b64encode(snippetImage.read()).decode(),
                    "mediaUploadValueContentType": "image/png",
                }
            ]

        if ytVideo:
            data["content"] = None
            data["mediaType"] = 103
            data["mediaValue"] = ytVideo

        if file:
            data["content"] = None
            if fileType == "audio":
                data["type"] = 2
                data["mediaType"] = 110

            elif fileType == "image":
                data["mediaType"] = 100
                data["mediaUploadValueContentType"] = "image/jpg"
                data["mediaUhqEnabled"] = False

            elif fileType == "gif":
                data["mediaType"] = 100
                data["mediaUploadValueContentType"] = "image/gif"
                data["mediaUhqEnabled"] = False

            else:
                raise TypeError(fileType)

            data["mediaUploadValue"] = b64encode(file.read()).decode()
            data["attachedObject"] = None
            data["extensions"] = None

        req = self.postRequest(f"/g/s/chat/thread/{chatId}/message", data)
        return Json(req)

    def get_community_info(self, comId: str):
        link = (
            f"/g/s-x{comId}/community/info"
            f"?withInfluencerList=1"
            f"&withTopicList=true"
            f"&influencerListOrderStrategy=fansCount"
        )
        req = self.getRequest(link)
        return Community(req["community"]).Community

    def mark_as_read(self, chatId: str):
        req = self.postRequest(f"/g/s/chat/thread/{chatId}/mark-as-read")
        return Json(req)

    def delete_message(self, messageId: str, chatId: str):
        req = self.deleteRequest(f"/g/s/chat/thread/{chatId}/message/{messageId}")
        return Json(req)

    def get_chat_messages(self, chatId: str, size: int = 25):
        req = self.getRequest(
            f"/g/s/chat/thread/{chatId}/message?v=2&pagingType=t&size={size}"
        )
        return GetMessages(req["messageList"]).GetMessages

    def get_message_info(self, messageId: str, chatId: str):
        req = self.getRequest(f"/g/s/chat/thread/{chatId}/message/{messageId}")
        return Message(req["message"]).Message

    def tip_coins(
        self,
        chatId: str = None,
        blogId: str = None,
        coins: int = 0,
        transactionId: str = None,
    ):
        if transactionId is None:
            transactionId = str(UUID(hexlify(os.urandom(16)).decode("ascii")))
        data = {
            "coins": coins,
            "tippingContext": {"transactionId": transactionId},
            "timestamp": int(timestamp() * 1000),
        }

        if chatId:
            link = f"/g/s/blog/{chatId}/tipping"
        elif blogId:
            link = f"/g/s/blog/{blogId}/tipping"
        else:
            raise TypeError("please put chat or blog Id")

        req = self.postRequest(link, data)
        return Json(req)

    def reset_password(
        self, email: str, password: str, code: str, deviceId: str = None
    ):
        if deviceId is None:
            deviceId = self.deviceId

        data = {
            "updateSecret": f"0 {password}",
            "emailValidationContext": {
                "data": {"code": code},
                "type": 1,
                "identity": email,
                "level": 2,
                "deviceID": deviceId,
            },
            "phoneNumberValidationContext": None,
            "deviceID": deviceId,
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest("/g/s/auth/reset-password", data)
        return Json(req)

    def change_password(self, password: str, newPassword: str):
        data = {
            "secret": f"0 {password}",
            "updateSecret": f"0 {newPassword}",
            "validationContext": None,
            "deviceID": self.deviceId,
        }

        req = self.postRequest("/g/s/auth/change-password", data)
        return Json(req)

    def get_user_info(self, userId: str):
        req = self.getRequest(f"/g/s/user-profile/{userId}")
        return UserProfile(req["userProfile"]).UserProfile

    def comment(self, comment: str, userId: str = None, replyTo: str = None):
        data = {
            "content": comment,
            "stickerId": None,
            "type": 0,
            "eventSource": "UserProfileView",
            "timestamp": int(timestamp() * 1000),
        }

        if replyTo:
            data["respondTo"] = replyTo
        req = self.postRequest(f"/g/s/user-profile/{userId}/g-comment", data)
        return Json(req)

    def delete_comment(self, userId: str = None, commentId: str = None):
        req = self.deleteRequest(f"/g/s/user-profile/{userId}/g-comment/{commentId}")
        return Json(req)

    def invite_by_host(self, chatId: str, userId: Union[str, list]):
        data = {"uidList": userId, "timestamp": int(timestamp() * 1000)}
        req = self.postRequest(f"/g/s/chat/thread/{chatId}/avchat-members", data)
        return Json(req)

    def kick(self, chatId: str, userId: str, rejoin: bool = True):
        rejoin = 1 if rejoin else 0
        req = self.deleteRequest(
            f"/g/s/chat/thread/{chatId}/member/{userId}?allowRejoin={rejoin}"
        )
        return Json(req)

    def get_invise_users(self, master_type = "newest", start: int = 0, size: int = 25):

        """
            function for masteringInvite:
                master_type:
                    new/newest : invited new usres from all ndcs or
                    online : invited online users from all ndcs
        """

        req = self.getRequest(
            f"/g/s/user-profile?type={master_type}&start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    def invise_invite(self, chatId: str, userId: Union[str, list]):

        """
            Invite users in your room global
                arguments:
                    userId: id object for users
                    chatId: id object for chats

                use as:
                    chat_info = client.get_from_link(input("chat link:  "))
                    chat_id = chat_info.objectId
                    # need function for invite user objext

      """

        if isinstance(userId, str): userIds = [userId]
        elif isinstance(userId, list): userIds = userId
        else: raise exceptions.WrongType

        data = {
            "uids": userIds,
            "timestamp": int(timestamp() * 1000)
        }

        req = self.postRequest(f"/g/s/chat/thread/{chatId}/member/invite", data)
        return Json(req)

    def block(self, userId: str):
        req = self.postRequest(f"/g/s/block/{userId}")
        return Json(req)

    def unblock(self, userId: str):
        req = self.deleteRequest(f"/g/s/block/{userId}")
        return Json(req)

    def get_public_chats(
        self, filterType: str = "recommended", start: int = 0, size: int = 50
    ):
        req = self.getRequest(
            f"/g/s/chat/thread?type=public-all&filterType={filterType}&start={start}&size={size}"
        )
        return ThreadList(req["threadList"]).ThreadList

    def get_content_modules(self, version: int = 2):
        req = self.getRequest(f"/g/s/home/discover/content-modules?v={version}")
        return Json(req)

    def get_banner_ads(self, size: int = 25, pagingType: str = "t"):
        link = (
            f"/g/s/topic/0/feed/banner-ads"
            f"?moduleId=711f818f-da0c-4aa7-bfa6-d5b58c1464d0&adUnitId=703798"
            f"&size={size}"
            f"&pagingType={pagingType}"
        )

        req = self.getRequest(link)
        return ItemList(req["itemList"]).ItemList

    def get_announcements(self, lang: str = "en", start: int = 0, size: int = 20):
        req = self.getRequest(
            f"/g/s/announcement?language={lang}&start={start}&size={size}"
        )
        return BlogList(req["blogList"]).BlogList

    def get_public_ndc(self, content_language: str = "en", size: int = 25):
        """
             for content_language:
                 content_language:
                     en:  english communities
                     de:  deuth communities
                     ru:  russian communities
                     es: espaniol communities
                     pt: portuguse communities
                     fr: francias communities
                     ar:  arabic communities

                   """
        req = self.getRequest(f"/g/s/topic/0/feed/community?language={content_language}&type=web-explore&categoryKey=recommendation&size={size}&pagingType=t")
        return CommunityList(req["communityList"]).CommunityList

    def search_community(
        self, word: str, lang: str = "en", start: int = 0, size: int = 25
    ):
        req = self.getRequest(
            f"/g/s/community/search?q={word}&language={lang}&completeKeyword=1&start={start}&size={size}"
        )
        return CommunityList(req["communityList"]).CommunityList

    def invite_to_voice_chat(self, userId: str = None, chatId: str = None):
        data = {"uid": userId, "timestamp": int(timestamp() * 1000)}
        req = self.postRequest(
            f"/g/s/chat/thread/{chatId}/vvchat-presenter/invite", data
        )
        return Json(req)

    def get_wallet_history(self, start: int = 0, size: int = 25):
        req = self.getRequest(f"/g/s/wallet/coin/history?start={start}&size={size}")
        return WalletHistory(req).WalletHistory

    def get_wallet_info(self):
        req = self.getRequest("/g/s/wallet")
        return WalletInfo(req["wallet"]).WalletInfo

    def get_all_users(self, usersType: str = "recent", start: int = 0, size: int = 25):
        """
        Types:
            - recent
            - banned
            - featured
            - leaders
            - curators
        """
        req = self.getRequest(
            f"/g/s/user-profile?type={usersType}&start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    def get_chat_members(self, start: int = 0, size: int = 25, chatId: str = None):
        req = self.getRequest(
            f"/g/s/chat/thread/{chatId}/member?start={start}&size={size}&type=default&cv=1.2"
        )
        return UserProfileList(req["memberList"]).UserProfileList

    def get_from_id(
        self, objectId: str, comId: str = None, objectType: int = 2
    ):  # never tried
        data = {
            "objectId": objectId,
            "targetCode": 1,
            "objectType": objectType,
            "timestamp": int(timestamp() * 1000),
        }

        link = f"/g/s/link-resolution"

        if comId:
            link = f"/g/s-x{comId}/link-resolution"

        req = self.postRequest(link, data)
        return FromCode(req["linkInfoV2"]["extensions"]["linkInfo"]).FromCode

    def chat_settings(
        self,
        chatId: str,
        viewOnly: bool = None,
        doNotDisturb: bool = None,
        canInvite: bool = False,
        canTip: bool = None,
        pin: bool = None,
    ):
        res = []

        if doNotDisturb:
            if doNotDisturb:
                opt = 2
            else:
                opt = 1
            data = {"alertOption": opt, "timestamp": int(timestamp() * 1000)}
            req = self.postRequest(
                f"/g/s/chat/thread/{chatId}/member/{self.uid}/alert", data
            )
            res.append(Json(req))

        if viewOnly is not None:
            if viewOnly:
                viewOnly = "enable"
            else:
                viewOnly = "disable"
            req = self.postRequest(f"/g/s/chat/thread/{chatId}/view-only/{viewOnly}")
            res.append(Json(req))

        if canInvite:
            if canInvite:
                canInvite = "enable"
            else:
                canInvite = "disable"
            req = self.postRequest(
                f"/g/s/chat/thread/{chatId}/members-can-invite/{canInvite}"
            )
            res.append(Json(req))

        if canTip:
            if canTip:
                canTip = "enable"
            else:
                canTip = "disable"
            req = self.postRequest(
                f"/g/s/chat/thread/{chatId}/tipping-perm-status/{canTip}"
            )
            res.append(Json(req))

        if pin:
            if pin:
                pin = "pin"
            else:
                pin = "unpin"
            req = self.postRequest(f"/g/s/chat/thread/{chatId}/{pin}")
            res.append(Json(req))

        return res

    def like_comment(self, commentId: str, userId: str = None, blogId: str = None):
        data = {"value": 4, "timestamp": int(timestamp() * 1000)}

        if userId:
            link = (
                f"/g/s/user-profile/{userId}/comment/{commentId}/g-vote?cv=1.2&value=1"
            )
        elif blogId:
            link = f"/g/s/blog/{blogId}/comment/{commentId}/g-vote?cv=1.2&value=1"
        else:
            raise TypeError("Please put blog or user Id")

        req = self.postRequest(link, data)
        return Json(req)

    def unlike_comment(self, commentId: str, blogId: str = None, userId: str = None):
        if userId:
            link = f"/g/s/user-profile/{userId}/comment/{commentId}/g-vote?eventSource=UserProfileView"
        elif blogId:
            link = f"/g/s/blog/{blogId}/comment/{commentId}/g-vote?eventSource=PostDetailView"
        else:
            raise TypeError("Please put blog or user Id")

        req = self.deleteRequest(link)
        return Json(req)

    def register(
        self, nickname: str, email: str, password: str, code: str, deviceId: str = None
    ):
        if deviceId is None:
            deviceId = self.deviceId

        data = {
            "secret": f"0 {password}",
            "deviceID": deviceId,
            "email": email,
            "clientType": 100,
            "nickname": nickname,
            "latitude": 0,
            "longitude": 0,
            "address": None,
            "clientCallbackURL": "narviiapp://relogin",
            "validationContext": {"data": {"code": code}, "type": 1, "identity": email},
            "type": 1,
            "identity": email,
            "timestamp": int(timestamp() * 1000),
        }

        req = self.postRequest("/g/s/auth/register", data)
        return Json(req)

    def remove_host(self, chatId: str, userId: str):
        req = self.deleteRequest(f"/g/s/chat/thread/{chatId}/co-host/{userId}")
        return Json(req)

    def edit_comment(self, commentId: str, comment: str, userId: str):
        data = {"content": comment, "timestamp": int(timestamp() * 1000)}
        req = self.postRequest(f"/g/s/user-profile/{userId}/comment/{commentId}", data)
        return Comment(req).Comments

    def get_comment_info(self, commentId: str, userId: str):
        req = self.getRequest(f"/g/s/user-profile/{userId}/comment/{commentId}")
        return Comment(req).Comments

    def get_notifications(self, size: int = 25, pagingType: str = "t"):
        req = self.getRequest(f"/g/s/notification?pagingType={pagingType}&size={size}")
        return NotificationList(req).NotificationList

    def get_notices(
        self, start: int = 0, size: int = 25, noticeType: str = "usersV2", status: int = 1
    ):
        req = self.getRequest(
            f"/g/s/notice?type={noticeType}&status={status}&start={start}&size={size}"
        )
        return NoticeList(req).NoticeList

    def accept_promotion(self, requestId: str):
        req = self.postRequest(f"/g/s/notice/{requestId}/accept")
        return Json(req)

    def decline_promotion(self, requestId: str):
        req = self.postRequest(f"/g/s/notice/{requestId}/decline")
        return Json(req)



import os
from base64 import b64encode
from binascii import hexlify
from time import time as timestamp
from time import timezone as timezones
from typing import Union, BinaryIO
from uuid import UUID

from ..lib.objects import *
from ..lib.async_sessions import AsyncSession as Session
from .acm import AsyncAcm as Acm


class AsyncSubClient(Acm, Session):
    def __init__(self, comId: str, proxies: dict = None, acm: bool = False):
        self.proxies = proxies
        self.comId = comId

        if acm:
            Acm.__init__(self, comId=self.comId, proxies=self.proxies)
        else:
            Session.__init__(self, proxies=self.proxies)

    async def get_video_rep_info(self, chatId: str):
        req = await self.getRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/avchat-reputation"
        )
        return RepInfo(req)

    async def claim_video_rep(self, chatId: str):
        req = await self.postRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/avchat-reputation"
        )
        return RepInfo(req)

    async def join_chat(self, chatId: str = None):
        req = await self.postRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.uid}"
        )
        return Json(req)

    async def upload_media(self, file: BinaryIO, fileType: str):
        if fileType == "audio":
            fileType = "audio/aac"
        elif fileType == "image":
            fileType = "image/jpg"
        else:
            raise TypeError(fileType)

        data = file.read()
        newHeaders = {"content-type": fileType, "content-length": str(len(data))}

        req = await self.postRequest("/g/s/media/upload", data=data, newHeaders=newHeaders)
        return req["mediaValue"]

    async def leave_chat(self, chatId: str = None):
        req = await self.deleteRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.uid}"
        )
        return Json(req)

    async def get_member_following(self, userId: str = None, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/user-profile/{userId}/joined?start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    async def get_member_followers(self, userId: str = None, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/user-profile/{userId}/member?start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    async def get_chat_threads(self, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/chat/thread?type=joined-me&start={start}&size={size}"
        )
        return ThreadList(req["threadList"]).ThreadList

    async def get_member_visitors(self, userId: str, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/user-profile/{userId}/visitors?start={start}&size={size}"
        )
        return VisitorsList(req["visitors"]).VisitorsList

    async def get_chat_messages(self, chatId: str, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/message?v=2&pagingType=t&size={size}"
        )
        return MessageList(req["messageList"]).MessageList

    async def get_user_info(self, userId: str):
        req = await self.getRequest(f"/x{self.comId}/s/user-profile/{userId}")
        return UserProfile(req["userProfile"]).UserProfile

    async def get_user_blogs(self, userId: str, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/blog?type=user&q={userId}&start={start}&size={size}"
        )
        return BlogList(req["blogList"]).BlogList

    async def get_user_wikis(self, userId: str, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/item?type=user-all&start={start}&size={size}&cv=1.2&uid={userId}"
        )
        return WikiList(req["itemList"]).WikiList

    async def get_all_users(self, usersType: str = "recent", start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/user-profile?type={usersType}&start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    async def get_chat_members(self, start: int = 0, size: int = 25, chatId: str = None):
        req = await self.getRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/member?start={start}&size={size}&type=default&cv=1.2"
        )
        return UserProfileList(req["memberList"]).UserProfileList

    async def get_chat_info(self, chatId: str):
        req = await self.getRequest(f"/x{self.comId}/s/chat/thread/{chatId}")
        return Thread(req["thread"]).Thread

    async def get_online_users(self, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/live-layer?topic=ndtopic:x{self.comId}:online-members&start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    async def get_public_chats(
        self, filterType: str = "recommended", start: int = 0, size: int = 50
    ):
        req = await self.getRequest(
            f"/x{self.comId}/s/chat/thread?type=public-all&filterType={filterType}&start={start}&size={size}"
        )
        return ThreadList(req["threadList"]).ThreadList

    async def send_message(
        self,
        chatId: str,
        message: str = None,
        messageType: int = 0,
        file: BinaryIO = None,
        fileType: str = None,
        replyTo: str = None,
        mentionUserIds: Union[list, str] = None,
        stickerId: str = None,
        snippetLink: str = None,
        ytVideo: str = None,
        snippetImage: BinaryIO = None,
        embedId: str = None,
        embedType: int = None,
        embedLink: str = None,
        embedTitle: str = None,
        embedContent: str = None,
        embedImage: BinaryIO = None,
    ):

        if message is not None and file is None:
            message = message.replace("[@", "‎‏").replace("@]", "‬‭")

        mentions = []
        if mentionUserIds:
            if isinstance(mentionUserIds, list):
                for mention_uid in mentionUserIds:
                    mentions.append({"uid": mention_uid})
            mentions.append({"uid": mentionUserIds})

        if embedImage:
            if not isinstance(embedImage, str):
                embedImage = [[100, self.upload_image(embedImage, "image"), None]]
            embedImage = [[100, embedImage, None]]

        data = {
            "type": messageType,
            "content": message,
            "attachedObject": {
                "objectId": embedId,
                "objectType": embedType,
                "link": embedLink,
                "title": embedTitle,
                "content": embedContent,
                "mediaList": embedImage,
            },
            "extensions": {"mentionedArray": mentions},
            "clientRefId": int(timestamp() / 10 % 100000000),
            "timestamp": int(timestamp() * 1000),
        }

        if replyTo:
            data["replyMessageId"] = replyTo

        if stickerId:
            data["content"] = None
            data["stickerId"] = stickerId
            data["type"] = 3

        if snippetLink and snippetImage:
            data["attachedObject"] = None
            data["extensions"]["linkSnippetList"] = [
                {
                    "link": snippetLink,
                    "mediaType": 100,
                    "mediaUploadValue": b64encode(snippetImage.read()).decode(),
                    "mediaUploadValueContentType": "image/png",
                }
            ]

        if ytVideo:
            data["content"] = None
            data["mediaType"] = 103
            data["mediaValue"] = ytVideo

        if file:
            data["content"] = None

            if fileType == "audio":
                data["type"] = 2
                data["mediaType"] = 110

            elif fileType == "image":
                data["mediaType"] = 100
                data["mediaUploadValueContentType"] = "image/jpg"
                data["mediaUhqEnabled"] = False

            elif fileType == "gif":
                data["mediaType"] = 100
                data["mediaUploadValueContentType"] = "image/gif"
                data["mediaUhqEnabled"] = False

            else:
                raise TypeError(fileType)

            data["mediaUploadValue"] = b64encode(file.read()).decode()
            data["attachedObject"] = None
            data["extensions"] = None

        req = await self.postRequest(f"/x{self.comId}/s/chat/thread/{chatId}/message", data)
        return Json(req)

    async def send_web_message(
        self,
        chatId: str,
        message: str = None,
        messageType: int = 0,
        icon: str = None,
        comId: str = None,
    ):
        if comId:
            self.comId = comId

        data = {
            "ndcId": f"x{self.comId}",
            "threadId": chatId,
            "message": {
                "content": message,
                "mediaType": 0,
                "type": messageType,
                "sendFailed": False,
                "clientRefId": 0,
            },
        }

        if icon:
            data["message"]["content"] = None
            data["message"]["uploadId"] = 0
            data["message"]["mediaType"] = 100
            data["message"]["mediaValue"] = icon

        return Json(await self.postRequest("/add-chat-message", data, webRequest=True))

    async def unfollow(self, userId: str):
        req = await self.postRequest(
            f"/x{self.comId}/s/user-profile/{userId}/member/{self.uid}"
        )
        return Json(req)

    async def follow(self, userId: Union[str, list]):
        data = {"timestamp": int(timestamp() * 1000)}

        if isinstance(userId, str):
            link = f"/x{self.comId}/s/user-profile/{userId}/member"
        elif isinstance(userId, list):
            link = f"/x{self.comId}/s/user-profile/{self.uid}/joined"
            data["targetUidList"] = userId

        req = await self.postRequest(link, data)
        return Json(req)

    async def start_chat(
        self,
        userId: Union[str, list],
        title: str = None,
        message: str = None,
        content: str = None,
        chatType: int = 0,
    ):
        if isinstance(userId, str):
            userId = [userId]

        data = {
            "title": title,
            "inviteeUids": userId,
            "initialMessageContent": message,
            "content": content,
            "type": chatType,
            "timestamp": int(timestamp() * 1000),
        }

        req = await self.postRequest(f"/x{self.comId}/s/chat/thread", data)
        return Thread(req['thread']).Thread

    async def invite_to_chat(self, userId: Union[str, list], chatId: str = None):
        if isinstance(userId, str):
            userId = [userId]

        data = {"uids": userId, "timestamp": int(timestamp() * 1000)}

        req = await self.postRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/member/invite", data=data
        )
        return Json(req)

    async def edit_profile(
        self,
        nickname: str = None,
        content: str = None,
        icon: BinaryIO = None,
        backgroundColor: str = None,
        backgroundImage: str = None,
        defaultBubbleId: str = None,
    ):
        data = {
            "address": None,
            "latitude": 0,
            "longitude": 0,
            "mediaList": None,
            "eventSource": "UserProfileView",
            "timestamp": int(timestamp() * 1000),
        }

        if content:
            data["content"] = content
        if nickname:
            data["nickname"] = nickname
        if icon:
            data["icon"] = self.upload_media(icon, "image")
        if defaultBubbleId:
            data["extensions"] = {"defaultBubbleId": defaultBubbleId}
        if backgroundColor:
            data["extensions"]["style"]["backgroundColor"] = backgroundColor
        if backgroundImage:
            data["extensions"]["style"]["backgroundMediaList"] = [
                [100, backgroundImage, None, None, None]
            ]

        req = await self.postRequest(f"/x{self.comId}/s/user-profile/{self.uid}", data)
        return Json(req)

    async def edit_chat(
        self,
        chatId: str,
        title: str = None,
        content: str = None,
        icon: str = None,
        background: str = None,
        keywords: list = None,
        announcement: str = None,
        pinAnnouncement: bool = None,
    ):
        res, data = [], {"timestamp": int(timestamp() * 1000)}

        if title:
            data["title"] = title
        if content:
            data["content"] = content
        if icon:
            data["icon"] = icon
        if keywords:
            data["keywords"] = keywords
        if announcement:
            data["extensions"]["announcement"] = announcement
        if pinAnnouncement:
            data["extensions"]["pinAnnouncement"] = pinAnnouncement
        if background:
            data = {
                "media": [100, background, None],
                "timestamp": int(timestamp() * 1000),
            }
            req = await self.postRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.uid}/background",
                data,
            )
            res.append(Json(req))

        req = await self.postRequest(f"/x{self.comId}/s/chat/thread/{chatId}", data)
        res.append(Json(req))

        return res

    async def chat_settings(
        self,
        chatId: str,
        viewOnly: bool = False,
        doNotDisturb: bool = True,
        canInvite: bool = False,
        canTip: bool = None,
        pin: bool = None,
        coHosts: Union[str, list] = None,
    ):
        res = []

        if doNotDisturb:
            if doNotDisturb:
                opt = 2
            else:
                opt = 1

            data = {"alertOption": opt, "timestamp": int(timestamp() * 1000)}
            req = await self.postRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.uid}/alert", data
            )
            res.append(Json(req))

        if viewOnly:
            if viewOnly:
                viewOnly = "enable"
            else:
                viewOnly = "disable"

            req = await self.postRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/view-only/{viewOnly}"
            )
            res.append(Json(req))

        if canInvite:
            if canInvite:
                canInvite = "enable"
            else:
                canInvite = "disable"

            req = await self.postRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/members-can-invite/{canInvite}"
            )
            res.append(Json(req))

        if canTip:
            if canTip:
                canTip = "enable"
            else:
                canTip = "disable"
            req = await self.postRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/tipping-perm-status/{canTip}"
            )
            res.append(Json(req))

        if pin:
            if pin:
                pin = "pin"
            else:
                pin = "unpin"

            req = await self.postRequest(f"/x{self.comId}/s/chat/thread/{chatId}/{pin}")
            res.append(Json(req))

        if coHosts:
            data = {"uidList": coHosts, "timestamp": int(timestamp() * 1000)}
            req = await self.postRequest(f"{self.comId}/s/chat/thread/{chatId}/co-host", data)
            res.append(Json(req))

        return res

    async def like_blog(self, blogId: str = None, wikiId: str = None):
        data = {"value": 4, "timestamp": int(timestamp() * 1000)}

        if blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/vote?cv=1.2&value=4"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/vote?cv=1.2&value=4"
        else:
            raise TypeError("Please put wiki or blog Id")

        req = await self.postRequest(link, data)
        return Json(req)

    async def unlike_blog(self, blogId: str = None, wikiId: str = None):
        if blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/vote?eventSource=FeedList"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/vote?eventSource=FeedList"
        else:
            raise TypeError("Please put wikiId or blogId")

        req = await self.deleteRequest(link)
        return Json(req)

    async def change_titles(self, userId: str, titles: list, colors: list):
        t = []
        for title, color in zip(titles, colors):
            t.append({"title": title, "color": color})
        data = {
            "adminOpName": 207,
            "adminOpValue": {"titles": t},
            "timestamp": int(timestamp() * 1000),
        }

        req = await self.postRequest(f"/x{self.comId}/s/user-profile/{userId}/admin", data)
        return Json(req)

    async def like_comment(
        self, commentId: str, blogId: str = None, wikiId: str = None, userId: str = None
    ):
        data = {"value": 1}

        if blogId:
            data["eventSource"] = "PostDetailView"
            link = f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?cv=1.2&value=1"
        elif wikiId:
            data["eventSource"] = "PostDetailView"
            link = f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/vote?cv=1.2&value=1"
        elif userId:
            data["eventSource"] = "UserProfileView"
            link = f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/vote?cv=1.2&value=1"
        else:
            raise TypeError("Please put a wiki or user or blog Id")

        req = await self.postRequest(link, data)
        return Json(req)

    async def unlike_comment(
        self, commentId: str, blogId: str = None, wikiId: str = None, userId: str = None
    ):
        if blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/g-vote?eventSource=PostDetailView"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/g-vote?eventSource=PostDetailView"
        elif userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/g-vote?eventSource=UserProfileView"
        else:
            raise TypeError("Please put a wiki or user or blog Id")

        req = await self.deleteRequest(link)
        return Json(req)

    async def comment(
        self,
        comment: str,
        userId: str = None,
        blogId: str = None,
        wikiId: str = None,
        replyTo: str = None,
        isGuest: bool = False,
    ):
        data = {"content": comment, "timestamp": int(timestamp() * 1000)}

        if replyTo:
            data["respondTo"] = replyTo
        if isGuest:
            comType = "g-comment"
        else:
            comType = "comment"

        if userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/{comType}"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/{comType}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/{comType}"
        else:
            raise TypeError("Please put a wiki or user or blog Id")

        req = await self.postRequest(link, data)
        return Json(req)

    async def delete_comment(
        self, commentId: str, userId: str = None, blogId: str = None, wikiId: str = None
    ):
        if userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}"
        else:
            raise TypeError("Please put blog or wiki or user Id")

        req = await self.deleteRequest(link)
        return Json(req)

    async def edit_comment(
        self,
        commentId: str,
        comment: str,
        userId: str = None,
        blogId: str = None,
        wikiId: str = None,
        replyTo: str = None,
        isGuest: bool = False,
    ):
        data = {"content": comment, "timestamp": int(timestamp() * 1000)}
        if replyTo:
            data["respondTo"] = replyTo
        if isGuest:
            comType = "g-comment"
        else:
            comType = "comment"

        if userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/{comType}/{commentId}"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/{comType}/{commentId}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/{comType}/{commentId}"
        else:
            raise TypeError("Please put blog or wiki or user Id")

        req = await self.postRequest(link, data)
        return Comment(req).Comments

    async def get_comment_info(
        self,
        commentId: str,
        userId: str = None,
        blogId: str = None,
        wikiId: str = None,
        isGuest: bool = False,
    ):
        if isGuest:
            comType = "g-comment"
        else:
            comType = "comment"

        if userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/{comType}/{commentId}"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/{comType}/{commentId}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/{comType}/{commentId}"
        else:
            raise TypeError("Please put blog or wiki or user Id")

        req = await self.getRequest(link)
        return Comment(req).Comments

    async def get_wall_comments(
        self, userId: str, sorting: str = "newest", start: int = 0, size: int = 25
    ):
        sorting = sorting.lower().replace("top", "vote")
        if sorting not in ["newest", "oldest", "vote"]:
            raise TypeError("حط تايب يا حمار")

        req = await self.getRequest(
            f"/x{self.comId}/s/user-profile/{userId}/comment?sort={sorting}&start={start}&size={size}"
        )
        return CommentList(req["commentList"]).CommentList

    async def get_blog_comments(
        self,
        wikiId: str = None,
        blogId: str = None,
        quizId: str = None,
        sorting: str = "newest",
        size: int = 25,
        start: int = 0,
    ):
        sorting = sorting.lower().replace("top", "vote")
        if sorting not in ["newest", "oldest", "vote"]:
            raise TypeError("حط تايب يا حمار")

        if quizId:
            blogId = quizId
        if blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/comment?sort={sorting}&start={start}&size={size}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/comment?sort={sorting}&start={start}&size={size}"
        else:
            raise TypeError("Please choose a wiki or a blog")

        req = await self.getRequest(link)
        return CommentList(req["commentList"]).CommentList

    async def vote_comment(self, blogId: str, commentId: str, value: bool = True):
        if value:
            value = 1
        else:
            value = -1

        data = {"value": value, "timestamp": int(timestamp() * 1000)}
        req = await self.postRequest(
            f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?cv=1.2&value=1",
            data,
        )
        return Json(req)

    async def vote_poll(self, blogId: str, optionId: str):
        data = {"value": 1, "timestamp": int(timestamp() * 1000)}
        req = await self.postRequest(
            f"/x{self.comId}/s/blog/{blogId}/poll/option/{optionId}/vote", data
        )
        return Json(req)

    async def get_blog_info(
        self, blogId: str = None, wikiId: str = None, folderId: str = None
    ):
        if blogId:
            link = f"/x{self.comId}/s/blog/{blogId}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}"
        elif folderId:
            link = f"/x{self.comId}/s/shared-folder/files/{folderId}"
        else:
            raise TypeError("Please put a wiki or blog Id")

        req = await self.getRequest(link)
        return GetInfo(req).GetInfo

    async def get_blogs(self, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/feed/featured?start={start}&size={size}"
        )
        return BlogList(req["featuredList"]).BlogList

    async def get_blogs_more(self, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/feed/featured-more?start={start}&size={size}"
        )
        return BlogList(req["blogList"]).BlogList

    async def get_blogs_all(self, start: int = 0, size: int = 25, pagingType: str = "t"):
        req = await self.getRequest(
            f"/x{self.comId}/s/feed/blog-all?pagingType={pagingType}&start={start}&size={size}"
        )
        return RecentBlogs(req["blogList"]).RecentBlogs

    async def tip_coins(
        self,
        coins: int,
        chatId: str = None,
        blogId: str = None,
        wikiId: str = None,
        transactionId: str = None,
    ):
        if transactionId is None:
            transactionId = str(UUID(hexlify(os.urandom(16)).decode("ascii")))
        data = {
            "coins": int(coins),
            "tippingContext": {"transactionId": transactionId},
            "timestamp": int(timestamp() * 1000),
        }

        if chatId:
            link = f"/x{self.comId}/s/chat/thread/{chatId}/tipping"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/tipping"
        elif wikiId:
            link = f"/x{self.comId}/s/tipping"
            data["objectType"] = 2
            data["objectId"] = wikiId
        else:
            raise TypeError("Please put a wiki or chat or blog Id")

        req = await self.postRequest(link, data)
        return Json(req)

    async def check_in(self, timezone: int = 180):
        data = {"timezone": timezone, "timestamp": int(timestamp() * 1000)}
        req = await self.postRequest(f"/x{self.comId}/s/check-in", data)
        return Json(req)

    async def check_in_lottery(self, timezone: int = 180):
        data = {"timezone": timezone, "timestamp": int(timestamp() * 1000)}
        req = await self.postRequest(f"/x{self.comId}/s/check-in/lottery", data)
        return Json(req)

    async def delete_message(
        self, chatId: str, messageId: str, asStaff: bool = False, reason: str = None
    ):
        data = {"adminOpName": 102, "timestamp": int(timestamp() * 1000)}

        if asStaff and reason:
            data["adminOpNote"] = {"content": reason}
            req = await self.postRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/message/{messageId}/admin", data
            )
        else:
            req = await self.deleteRequest(
                f"/x{self.comId}/s/chat/thread/{chatId}/message/{messageId}"
            )

        return Json(req)

    async def invite_by_host(self, chatId: str, userId: Union[str, list]):
        req = await self.postRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/member/{userId}/invite-av-chat"
        )
        return Json(req)

    async def strike(self, userId: str, time: str, title: str = None, reason: str = None):
        times = {
            "1-Hours": 3600,
            "3-Hours": 10800,
            "6-Hours": 21600,
            "12-Hours": 43200,
            "24-Hours": 86400,
        }.get(time, 3600)

        data = {
            "uid": userId,
            "title": title,
            "content": reason,
            "attachedObject": {"objectId": userId, "objectType": 0},
            "penaltyType": 1,
            "penaltyValue": times,
            "adminOpNote": {},
            "noticeType": 4,
            "timestamp": int(timestamp() * 1000),
        }

        req = await self.postRequest(f"/x{self.comId}/s/notice", data)
        return Json(req)

    async def ban(self, userId: str, reason: str, banType: int = None):
        data = {
            "reasonType": banType,
            "note": {"content": reason},
            "timestamp": int(timestamp() * 1000),
        }

        req = await self.postRequest(f"/x{self.comId}/s/user-profile/{userId}/ban", data)
        return Json(req)

    async def unban(self, userId: str, reason: str = "هذا العضو كان شاطر اخر كم يوم"):
        data = {"note": {"content": reason}, "timestamp": int(timestamp() * 1000)}
        req = await self.postRequest(f"/x{self.comId}/s/user-profile/{userId}/unban", data)
        return Json(req)

    async def hide(
        self,
        note: str = None,
        blogId: str = None,
        userId: str = None,
        wikiId: str = None,
        chatId: str = None,
    ):
        data = {
            "adminOpName": 18 if userId else 110,
            "adminOpValue": None if userId else 9,
            "timestamp": int(timestamp() * 1000),
        }

        if userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/admin"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/admin"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/admin"
        elif chatId:
            link = f"/x{self.comId}/s/chat/thread/{chatId}/admin"
        else:
            raise TypeError("Please put a wiki or user or chat or blog Id")
        if note:
            data["adminOpNote"] = {"content": note}

        req = await self.postRequest(link, data)
        return Json(req)

    async def unhide(
        self,
        note: str = None,
        blogId: str = None,
        userId: str = None,
        wikiId: str = None,
        chatId: str = None,
    ):
        data = {
            "adminOpName": 19 if userId else 110,
            "adminOpValue": 0,
            "timestamp": int(timestamp() * 1000),
        }

        if userId:
            link = f"/x{self.comId}/s/user-profile/{userId}/admin"
        elif blogId:
            link = f"/x{self.comId}/s/blog/{blogId}/admin"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}/admin"
        elif chatId:
            link = f"/x{self.comId}/s/chat/thread/{chatId}/admin"
        else:
            raise TypeError("Please put a wiki or user or chat or blog Id")
        if note:
            data["adminOpNote"] = {"content": note}

        req = await self.postRequest(link, data)
        return Json(req)

    async def send_warning(self, userId: str, reason: str = None):
        data = {
            "uid": userId,
            "title": "Custom",
            "content": reason,
            "attachedObject": {"objectId": userId, "objectType": 0},
            "penaltyType": 0,
            "adminOpNote": {},
            "noticeType": 7,
            "timestamp": int(timestamp() * 1000),
        }

        req = await self.postRequest(f"/x{self.comId}/s/notice", data)
        return Json(req)

    async def invite_to_voice_chat(self, userId: str = None, chatId: str = None):
        data = {"uid": userId, "timestamp": int(timestamp() * 1000)}
        req = await self.postRequest(
            f"/g/x{self.comId}/chat/thread/{chatId}/vvchat-presenter/invite", data
        )
        return Json(req)

    async def post_blog(self, title: str, content: str, fansOnly: bool = False):
        data = {
            "extensions": {"fansOnly": fansOnly},
            "content": content,
            "latitude": 0,
            "longitude": 0,
            "title": title,
            "type": 0,
            "contentLanguage": "ar",
            "eventSource": "GlobalComposeMenu",
            "timestamp": int(timestamp() * 1000),
        }

        req = await self.postRequest(f"/x{self.comId}/s/blog", data)
        return Json(req)

    async def post_wiki(
        self,
        title: str,
        content: str,
        fansOnly: bool = False,
        icon: str = None,
        backgroundColor: str = None,
        keywords: Union[str, list] = None,
    ):
        data = {
            "extensions": {
                "fansOnly": fansOnly,
                "props": [],
                "style": {"backgroundColor": backgroundColor},
            },
            "content": content,
            "keywords": keywords,
            "label": title,
            "latitude": 0,
            "longitude": 0,
            "eventSource": "UserProfileView",
            "timestamp": int(timestamp() * 1000),
        }
        if icon:
            data["icon"] = icon

        req = await self.postRequest(f"/x{self.comId}/s/item", data)
        return Json(req)

    async def delete_blog(self, blogId: str):
        req = await self.deleteRequest(f"/x{self.comId}/s/blog/{blogId}")
        return Json(req)

    async def delete_wiki(self, wikiId: str):
        req = await self.deleteRequest(f"/x{self.comId}/s/item/{wikiId}")
        return Json(req)

    async def activate_status(self, status: int = 1):
        data = {
            "onlineStatus": status,
            "duration": 86400,
            "timestamp": int(timestamp() * 1000),
        }
        req = await self.postRequest(
            f"/x{self.comId}/s/user-profile/{self.uid}/online-status", data
        )
        return Json(req)

    async def subscribe(self, userId: str, autoRenew: str = False, transactionId: str = None):
        if transactionId is None:
            transactionId = str(UUID(hexlify(os.urandom(16)).decode("ascii")))
        data = {
            "paymentContext": {
                "transactionId": transactionId,
                "isAutoRenew": autoRenew,
            },
            "timestamp": int(timestamp() * 1000),
        }

        req = await self.postRequest(f"/x{self.comId}/s/influencer/{userId}/subscribe", data)
        return Json(req)

    async def submit_wiki(self, wikiId: str, message: str = None):
        data = {
            "message": message,
            "itemId": wikiId,
            "timestamp": int(timestamp() * 1000),
        }

        req = await self.postRequest(f"/x{self.comId}/s/knowledge-base-request", data)
        return Json(req)

    async def edit_blog(
        self,
        title: str,
        content: str,
        blogId: str = None,
        wikiId: str = None,
        fansOnly: bool = False,
        backgroundColor: str = None,
        media: list = None,
    ):
        data = {
            "title": title,
            "content": content,
            "timestamp": int(timestamp() * 1000),
        }

        if media:
            data["mediaList"] = [[100, media, None, "XYZ", None, {"fileName": "Amino"}]]
        if fansOnly:
            data["extensions"]["fansOnly"] = True
        if backgroundColor:
            data["extensions"] = {"backgroundColor": backgroundColor}

        if blogId:
            link = f"/x{self.comId}/s/blog/{blogId}"
        elif wikiId:
            link = f"/x{self.comId}/s/item/{wikiId}"
        else:
            raise TypeError("Please put blogId or wikiId")

        req = await self.postRequest(link, data)
        return Json(req)

    async def get_chat_bubbles(self, start: int = 0, size: int = 20):
        req = await self.getRequest(
            f"/x{self.comId}/s/chat/chat-bubble?type=all-my-bubbles&start={start}&size={size}"
        )
        return BubbleList(req["chatBubbleList"]).BubbleList

    async def select_bubble(self, bubbleId: str, apply: int = 0, chatId: str = None):
        data = {
            "bubbleId": bubbleId,
            "applyToAll": apply,
            "timestamp": int(timestamp() * 1000),
        }
        if chatId:
            data["threadId"] = chatId

        req = await self.postRequest(f"/x{self.comId}/s/chat/thread/apply-bubble")
        return Json(req)

    async def delete_chat_bubble(self, bubbleId: str):
        req = await self.deleteRequest(url=f"/x{self.comId}/s/chat/chat-bubble/{bubbleId}")
        return Json(req)

    async def get_chat_bubble_templates(self, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/chat/chat-bubble/templates?start={start}&size={size}"
        )
        return BubbleTemplates(req["templateList"])

    async def upload_custom_bubble(self, templateId: str):
        req = await self.postRequest(
            f"/x{self.comId}/s/chat/chat-bubble/templates/{templateId}/generate"
        )
        return Json(req)

    async def kick(self, chatId: str, userId: str, rejoin: bool = True):
        if rejoin:
            rejoin = 1
        if not rejoin:
            rejoin = 0

        req = await self.deleteRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/member/{userId}?allowRejoin={rejoin}"
        )
        return Json(req)

    async def block(self, userId: str):
        req = await self.postRequest(f"/x{self.comId}/s/block/{userId}")
        return Json(req)

    async def flag(
        self,
        reason: str,
        flagType: str = "spam",
        userId: str = None,
        wikiId: str = None,
        blogId: str = None,
    ):
        types = {
            "violence": 106,
            "hate": 107,
            "suicide": 108,
            "troll": 109,
            "nudity": 110,
            "bully": 0,
            "off-topic": 4,
            "spam": 2,
        }

        data = {"message": reason, "timestamp": int(timestamp() * 1000), "flagType": types.get(flagType, 106)}

        if userId:
            data["objectId"] = userId
            data["objectType"] = 0
        elif blogId:
            data["objectId"] = blogId
            data["objectType"] = 1
        elif wikiId:
            data["objectId"] = wikiId
            data["objectType"] = 2
        else:
            raise TypeError("choose a certain type to report")

        req = await self.postRequest(f"/x{self.comId}/s/flag", data)
        return Json(req)

    async def send_active_time(self, tz: int = int(-timezones // 1000), timers: list = None):
        data = {
            "userActiveTimeChunkList": timers if timers else [{"start": int(timestamp()), "end": int(timestamp() + 300)}],
            "timestamp": int(timestamp() * 1000),
            "optInAdsFlags": 2147483647,
            "timezone": tz,
        }

        req = await self.postRequest(
            f"/x{self.comId}/s/community/stats/user-active-time", data, minify=True
        )
        return Json(req)

    async def transfer_host(self, chatId: str, userIds: list):
        data = {"uidList": userIds, "timestamp": int(timestamp() * 1000)}
        req = await self.postRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/transfer-organizer", data
        )
        return Json(req)

    async def accept_host(self, chatId: str, requestId: str):
        data = {"timestamp": int(timestamp() * 1000)}
        req = await self.postRequest(
            f"/x{self.comId}/s/chat/thread/{chatId}/transfer-organizer/{requestId}/accept",
            data,
        )
        return Json(req)

    async def set_cohost(self, chatId : str, asistent_id : [str, list]):
         data = {
            "uidList": asistent_id,
            "timestamp": int(timestamp() * 1000)
        }

         req = await self.postRequest(f"/x{self.comId}/s/chat/thread/{chatId}/co-host", data)
         return Json(req)


    async def del_cohost(self, chatId : str, userId : str):
           req = await self.deleteRequest(f"/x{self.comId}/s/chat/thread/{chatId}/co-host/{userId}")
           return Json(req)

    async def get_quizzes(self, quizzesType: str = "recent", start: int = 0, size: int = 25):
        link = {
            "recent": f"x{self.comId}/s/blog?type=quizzes-recent&start={start}&size={size}",
            "trending": f"x{self.comId}/s/feed/quiz-trending?start={start}&size={size}",
            "best": f"x{self.comId}/s/feed/quiz-best-quizzes?start={start}&size={size}",
        }.get(quizzesType)

        req = await self.getRequest(link)
        return BlogList(req["blogList"]).BlogList

    async def get_quiz_questions(self, quizId: str):
        req = await self.getRequest(f"/x{self.comId}/s/blog/{quizId}?action=review")
        return QuizQuestionList(req["blog"]["quizQuestionList"]).QuizQuestionList

    async def play_quiz(self, quizId: str, questions: list, answers: list, mode: int = 0):
        data = {
            "mode": mode,
            "quizAnswerList": [
                {"optIdList": [answer], "quizQuestionId": question, "timeSpent": 0.0}
                for answer, question in zip(answers, questions)
            ],
            "timestamp": int(timestamp() * 1000),
        }
        req = await self.postRequest(f"/x{self.comId}/s/blog/{quizId}/quiz/result", data)
        return req

    async def get_quiz_rankings(self, quizId: str, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/blog/{quizId}/quiz/result?start={start}&size={size}"
        )
        return QuizRankings(req).QuizRankings

    async def search_user(self, username: str, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/user-profile?type=name&q={username}&start={start}&size={size}"
        )
        return UserProfileList(req["userProfileList"]).UserProfileList

    async def search_blog(self, words: str, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/blog?type=keywords&q={words}&start={start}&size={size}"
        )
        return BlogList(req["blogList"]).BlogList

    async def get_notifications(self, size: int = 25, pagingType: str = "t"):
        req = await self.getRequest(
            f"/x{self.comId}/s/notification?pagingType={pagingType}&size={size}"
        )
        return NotificationList(req).NotificationList

    async def get_notices(
        self, start: int = 0, size: int = 25, noticeType: str = "usersV2", status: int = 1
    ):
        req = await self.getRequest(
            f"/x{self.comId}/s/notice?type={noticeType}&status={status}&start={start}&size={size}"
        )
        return NoticeList(req).NoticeList

    async def accept_promotion(self, requestId: str):
        if not isinstance(requestId, str):
            raise TypeError(f"Please use a string not {type(requestId)}")
        req = await self.postRequest(f"/x{self.comId}/s/notice/{requestId}/accept")
        return Json(req)

    async def decline_promotion(self, requestId: str):
        if not isinstance(requestId, str):
            raise TypeError(f"Please use a string not {type(requestId)}")
        req = await self.postRequest(f"/x{self.comId}/s/notice/{requestId}/decline")
        return Json(req)

    async def sendWebActive(self):
        data = {"ndcId": self.comId}
        return await self.postRequest(
            "/community/stats/web-user-active-time", data, webRequest=True
        )

    async def get_recent_blogs(self, pageToken: str = None, start: int = 0, size: int = 25):
        req = await self.getRequest(
            f"/x{self.comId}/s/feed/blog-all?pagingType=t&start={start}&size={size}"
        )
        return RecentBlogs(req["BlogList"]).RecentBlogs

    async def publish_to_featured(
        self,
        time: int,
        userId: str = None,
        chatId: str = None,
        blogId: str = None,
        wikiId: str = None,
    ):
        time = (
            {1: 3600, 2: 7200, 3: 10800}.get(time)
            if chatId
            else {1: 86400, 2: 172800, 3: 259200}.get(time)
        )

        data = {
            "adminOpName": 114,
            "adminOpValue": {"featuredDuration": time},
            "timestamp": int(timestamp() * 1000),
        }

        if userId:
            featuredType, endpoint = 4, f"user-profile/{userId}"
        elif blogId:
            featuredType, endpoint = 1, f"blog/{blogId}"
        elif wikiId:
            featuredType, endpoint = 1, f"item/{blogId}"
        elif chatId:
            featuredType, endpoint = 5, f"chat/thread/{chatId}"
        else:
            raise TypeError("Please Specify Object ID")

        data["adminOpValue"]["featuredType"] = featuredType
        req = await self.postRequest(f"/x{self.comId}/s/{endpoint}/admin", data)
        return Json(req)

    async def remove_from_featured(
        self,
        userId: str = None,
        chatId: str = None,
        blogId: str = None,
        wikiId: str = None,
    ):
        data = {
            "adminOpName": 114,
            "adminOpValue": {"featuredType": 0},
            "timestamp": int(timestamp() * 1000),
        }

        if userId:
            endpoint = f"user-profile/{userId}"
        elif blogId:
            endpoint = f"blog/{blogId}"
        elif wikiId:
            endpoint = f"item/{blogId}"
        elif chatId:
            endpoint = f"chat/thread/{chatId}"
        else:
            raise TypeError("Please Specify Object ID")

        req = await self.postRequest(f"/x{self.comId}/s/{endpoint}/admin", data)
        return Json(req)
